@@ -30,6 +30,7 @@ int printe(const PrintType type)
 				break;
 			case PT_Info:
 			case PT_Infoless:
+			case PT_Warning:
 			case PT_Error:
 			case PT_Config:
 			case PT_ShortMultiline:
@@ -47,6 +48,9 @@ int printe(const PrintType type)
 			case PT_Infoless:
 				printf("%s\n", errorstring);
 				break;
+			case PT_Warning:
+				printf("Warning: %s\n", errorstring);
+				break;
 			case PT_Error:
 				printf("Error: %s\n", errorstring);
 				break;
@@ -67,7 +71,7 @@ int printe(const PrintType type)
 
 int logprint(const PrintType type)
 {
-	/* buffer needs some extra space for timestamp + infor compared to errorstring */
+	/* buffer needs some extra space for timestamp + info compared to errorstring */
 	char timestamp[22], buffer[1060];
 	time_t current;
 	FILE *logfile;
@@ -93,6 +97,9 @@ int logprint(const PrintType type)
 			case PT_Infoless:
 				snprintf(buffer, 1060, "[%s] %s\n", timestamp, errorstring);
 				break;
+			case PT_Warning:
+				snprintf(buffer, 1060, "[%s] Warning: %s\n", timestamp, errorstring);
+				break;
 			case PT_Error:
 				snprintf(buffer, 1060, "[%s] Error: %s\n", timestamp, errorstring);
 				break;
@@ -122,6 +129,9 @@ int logprint(const PrintType type)
 		switch (type) {
 			case PT_Multiline:
 				break;
+			case PT_Warning:
+				syslog(LOG_WARNING, "Warning: %s", errorstring);
+				break;
 			case PT_Error:
 				syslog(LOG_ERR, "Error: %s", errorstring);
 				break;
@@ -277,22 +287,35 @@ char *getversion(void)
 	return versionbuffer;
 }
 
-void timeused(const char *func, const int reset)
+double timeused(const char *func, const int reset)
 {
 	static struct timeval starttime;
 	struct timeval endtime;
+	double used_secs;
 
+	if (reset) {
+		gettimeofday(&starttime, NULL);
+		return 0.0;
+	}
+
+	if (gettimeofday(&endtime, NULL) != 0) {
+		return 0.0;
+	}
+
+	used_secs = (double)(endtime.tv_usec - starttime.tv_usec) / 1000000 + (double)(endtime.tv_sec - starttime.tv_sec);
+
+	if (debug) {
+		printf("%s() in %f s\n", func, used_secs);
+	}
+
+	return used_secs;
+}
+
+void timeused_debug(const char *func, const int reset)
+{
 	if (!debug) {
 		return;
 	}
 
-	if (reset) {
-		gettimeofday(&starttime, NULL);
-		return;
-	}
-
-	if (gettimeofday(&endtime, NULL) != 0) {
-		return;
-	}
-	printf("%s() in %f s\n", func, (double)(endtime.tv_usec - starttime.tv_usec) / 1000000 + (double)(endtime.tv_sec - starttime.tv_sec));
+	timeused(func, reset);
 }
