@@ -33,6 +33,7 @@
 #include <sys/sysctl.h>
 #include <net/if.h>
 #include <ifaddrs.h>
+#define BSD_VNSTAT
 #endif
 
 /* OpenBSD and NetBSD don't support the ' character (decimal conversion) in printf formatting */
@@ -124,7 +125,7 @@ and most can be changed later from the config file.
 
 /* default interface */
 #ifndef DEFIFACE
-#define DEFIFACE "eth0"
+#define DEFIFACE ""
 #endif
 
 /* default locale */
@@ -231,6 +232,13 @@ and most can be changed later from the config file.
 #define LOGFILE "/var/log/vnstat/vnstat.log"
 #define PIDFILE "/var/run/vnstat/vnstat.pid"
 #define IS64BIT -2
+#define WALDB 0
+#define WALDBCHECKPOINTINTERVALMINS 240
+#define SLOWDBWARNLIMIT 4.0 // needs to be less than DBREADTIMEOUTSECS
+#define DBSYNCHRONOUS -1
+
+/* database read timeout */
+#define DBREADTIMEOUTSECS 5
 
 /* no transparency by default */
 #define TRANSBG 0
@@ -249,8 +257,10 @@ and most can be changed later from the config file.
 #define CTX "606060"
 #define CTXD "-"
 
-/* number of retries for non-fatal database errors */
-#define DBRETRYLIMIT 3
+/* number of retries after non-fatal database errors, */
+/* will result in given number + 1 tries in total before exit, */
+/* a full disk (as reported by sqlite) will no cause retries or exit */
+#define DBRETRYLIMIT 5
 
 /* internal config structure */
 typedef struct {
@@ -263,7 +273,7 @@ typedef struct {
 	char cline[8], clinel[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8];
 	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, sampletime, hourlyrate, summaryrate;
 	int32_t monthrotate, monthrotateyears, maxbw, spacecheck, trafficlessentries, transbg, ostyle;
-	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit;
+	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit, waldb, dbsynchronous;
 	char cfgfile[512], logfile[512], pidfile[512];
 	char daemonuser[33], daemongroup[33];
 	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, savestatus;
@@ -296,6 +306,7 @@ typedef struct ibwnode {
 typedef enum PrintType {
 	PT_Info = 0,
 	PT_Infoless,
+	PT_Warning,
 	PT_Error,
 	PT_Config,
 	PT_Multiline,
@@ -314,7 +325,8 @@ char *strncpy_nt(char *dest, const char *src, size_t n);
 int isnumeric(const char *s);
 void panicexit(const char *sourcefile, const int sourceline) __attribute__((noreturn));
 char *getversion(void);
-void timeused(const char *func, const int reset);
+double timeused(const char *func, const int reset);
+void timeused_debug(const char *func, const int reset);
 
 /* global variables */
 extern CFG cfg;
