@@ -1,5 +1,6 @@
 #include "common.h"
 #include "ifinfo.h"
+#include "iflist.h"
 #include "dbsql.h"
 #include "dbaccess.h"
 #include "datacache.h"
@@ -15,10 +16,6 @@ void daemonize(void)
 	int i;
 	char str[10];
 
-	if (getppid() == 1) {
-		return; /* already a daemon */
-	}
-
 	i = (int)fork();
 
 	if (i < 0) { /* fork error */
@@ -115,73 +112,76 @@ void daemonize(void)
 
 unsigned int addinterfaces(DSTATE *s)
 {
-	char *ifacelist, interface[32];
-	unsigned int index = 0, count = 0;
+	iflist *ifl = NULL, *ifl_iterator = NULL;
+	unsigned int count = 0;
 	uint32_t bwlimit = 0;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	/* get list of currently visible interfaces */
-	if (getiflist(&ifacelist, 0) == 0) {
-		free(ifacelist);
+	if (getiflist(&ifl, 0, 1) == 0) {
+		iflistfree(&ifl);
 		return 0;
 	}
 
-	if (strlen(ifacelist) < 2) {
-		free(ifacelist);
+	if (ifl == NULL) {
 		return 0;
 	}
 
-	if (debug)
-		printf("Interface list: \"%s\"\n", ifacelist);
-
-	while (sscanf(ifacelist + index, "%31s", interface) != EOF) {
-		if (debug)
-			printf("Processing: \"%s\"\n", interface);
-
-		index += strlen(interface) + 1;
-
-		/* skip local interfaces */
-		if ((strcmp(interface, "lo") == 0) || (strcmp(interface, "lo0") == 0) || (strcmp(interface, "sit0") == 0)) {
-			if (debug)
-				printf("skip\n");
-			continue;
+	if (debug) {
+		printf("Interface list:");
+		ifl_iterator = ifl;
+		while (ifl_iterator != NULL) {
+			printf(" \"%s\"", ifl_iterator->interface);
+			ifl_iterator = ifl_iterator->next;
 		}
+		printf("\n");
+	}
+
+	ifl_iterator = ifl;
+	while (ifl_iterator != NULL) {
+		if (debug)
+			printf("Processing: \"%s\"\n", ifl_iterator->interface);
 
 		/* skip already known interfaces */
-		if (db_getinterfacecountbyname(interface)) {
+		if (db_getinterfacecountbyname(ifl_iterator->interface)) {
 			if (debug)
 				printf("already known\n");
+			ifl_iterator = ifl_iterator->next;
 			continue;
 		}
 
 		/* create database for interface */
-		if (!db_addinterface(interface)) {
+		if (!db_addinterface(ifl_iterator->interface)) {
 			if (debug)
 				printf("add failed, skip\n");
-		}
-
-		if (!getifinfo(interface)) {
-			if (debug)
-				printf("getifinfo failed, skip\n");
-			/* remove empty entry from database since the interface can't provide data */
-			db_removeinterface(interface);
+			ifl_iterator = ifl_iterator->next;
 			continue;
 		}
 
-		db_setcounters(interface, ifinfo.rx, ifinfo.tx);
+		if (!getifinfo(ifl_iterator->interface)) {
+			if (debug)
+				printf("getifinfo failed, skip\n");
+			/* remove empty entry from database since the interface can't provide data */
+			db_removeinterface(ifl_iterator->interface);
+			ifl_iterator = ifl_iterator->next;
+			continue;
+		}
+
+		db_setcounters(ifl_iterator->interface, ifinfo.rx, ifinfo.tx);
 
 		count++;
-		ibwget(interface, &bwlimit);
+		ibwget(ifl_iterator->interface, &bwlimit);
 		if (bwlimit > 0) {
-			snprintf(errorstring, 1024, "Interface \"%s\" added with %" PRIu32 " Mbit bandwidth limit.", interface, bwlimit);
+			snprintf(errorstring, 1024, "Interface \"%s\" added with %" PRIu32 " Mbit bandwidth limit.", ifl_iterator->interface, bwlimit);
 		} else {
-			snprintf(errorstring, 1024, "Interface \"%s\" added. Warning: no bandwidth limit has been set.", interface);
+			snprintf(errorstring, 1024, "Interface \"%s\" added. Warning: no bandwidth limit has been set.", ifl_iterator->interface);
 		}
 		printe(PT_Infoless);
 		if (s->running) {
-			datacache_add(&s->dcache, interface, 1);
+			datacache_add(&s->dcache, ifl_iterator->interface, 1);
 		}
+		ifl_iterator = ifl_iterator->next;
 	}
 
 	if (count && !s->running) {
@@ -195,8 +195,8 @@ unsigned int addinterfaces(DSTATE *s)
 		printf("Unwanted interfaces can be removed from monitoring with \"vnstat --remove\".\n");
 	}
 
-	free(ifacelist);
-	timeused(__func__, 0);
+	iflistfree(&ifl);
+	timeused_debug(__func__, 0);
 	return count;
 }
 
@@ -240,6 +240,7 @@ void debugtimestamp(void)
 
 void initdstate(DSTATE *s)
 {
+	db = NULL;
 	noexit = 1;		   /* disable exits in functions */
 	debug = 0;		   /* debug disabled by default */
 	disableprints = 0; /* let prints be visible */
@@ -254,7 +255,6 @@ void initdstate(DSTATE *s)
 	s->alwaysadd = 0;
 	s->iflisthash = 0;
 	s->cfgfile[0] = '\0';
-	s->dirname[0] = '\0';
 	s->user[0] = '\0';
 	s->group[0] = '\0';
 	s->prevdbupdate = 0;
@@ -265,9 +265,10 @@ void initdstate(DSTATE *s)
 	s->cleanuphour = getcurrenthour();
 	s->dbretrycount = 0;
 	s->dcache = NULL;
+	s->prevwaldbcheckpoint = time(NULL);
 }
 
-void preparedatabases(DSTATE *s)
+void preparedatabase(DSTATE *s)
 {
 	s->dbifcount = db_getinterfacecount();
 
@@ -285,7 +286,7 @@ void preparedatabases(DSTATE *s)
 		exit(EXIT_FAILURE);
 	}
 
-	if (!spacecheck(s->dirname)) {
+	if (!spacecheck(cfg.dbdir)) {
 		printf("Error: Not enough free diskspace available, exiting.\n");
 		exit(EXIT_FAILURE);
 	}
@@ -295,9 +296,6 @@ void preparedatabases(DSTATE *s)
 			s->dbifcount = 0;
 			return;
 		}
-	}
-
-	if (s->dbifcount == 0) {
 		printf("No interfaces found in database, adding available interfaces...\n");
 	}
 
@@ -316,8 +314,8 @@ unsigned int importlegacydbs(DSTATE *s)
 	struct dirent *di;
 	unsigned int importcount = 0;
 
-	if ((dir = opendir(s->dirname)) == NULL) {
-		printf("Error: Unable to open database directory \"%s\": %s\n", s->dirname, strerror(errno));
+	if ((dir = opendir(cfg.dbdir)) == NULL) {
+		printf("Error: Unable to open database directory \"%s\": %s\n", cfg.dbdir, strerror(errno));
 		printf("Make sure it exists and is at least read enabled for current user.\n");
 		printf("Exiting...\n");
 		exit(EXIT_FAILURE);
@@ -325,12 +323,12 @@ unsigned int importlegacydbs(DSTATE *s)
 
 	s->dbifcount = 0;
 	while ((di = readdir(dir))) {
-		if ((di->d_name[0] != '.') && (strcmp(di->d_name, DATABASEFILE) != 0)) {
+		if ((di->d_name[0] != '.') && (strncmp(di->d_name, DATABASEFILE, strlen(DATABASEFILE)) != 0)) {
 			/* ignore already known interfaces */
 			if (db_getinterfacecountbyname(di->d_name)) {
 				continue;
 			}
-			if (importlegacydb(di->d_name, s->dirname)) {
+			if (importlegacydb(di->d_name, cfg.dbdir)) {
 				importcount++;
 			}
 		}
@@ -360,9 +358,9 @@ void setsignaltraps(void)
 
 void filldatabaselist(DSTATE *s)
 {
-	dbiflist *dbifl = NULL, *dbifl_iterator = NULL;
+	iflist *dbifl = NULL, *dbifl_iterator = NULL;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	if (db_getiflist(&dbifl) < 0) {
 		errorexitdaemon(s, 1);
@@ -383,7 +381,7 @@ void filldatabaselist(DSTATE *s)
 		dbifl_iterator = dbifl_iterator->next;
 	}
 
-	dbiflistfree(&dbifl);
+	iflistfree(&dbifl);
 	s->sync = 0;
 
 	/* disable update interval check for one loop if database list was refreshed */
@@ -397,7 +395,7 @@ void filldatabaselist(DSTATE *s)
 	} else {
 		s->updateinterval = 120;
 	}
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 }
 
 void adjustsaveinterval(DSTATE *s)
@@ -425,7 +423,7 @@ void processdatacache(DSTATE *s)
 {
 	datacache *iterator = s->dcache;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	while (iterator != NULL) {
 
@@ -464,7 +462,7 @@ void processdatacache(DSTATE *s)
 	if (s->bootdetected) {
 		s->bootdetected = 0;
 	}
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 
 	if (s->dodbsave) {
 		flushcachetodisk(s);
@@ -581,6 +579,7 @@ int processifinfo(DSTATE *s, datacache **dc)
 void flushcachetodisk(DSTATE *s)
 {
 	int ret;
+	double used_secs = 0.0;
 	uint32_t logcount = 0;
 	datacache *iterator = s->dcache;
 	xferlog *logiterator;
@@ -629,12 +628,6 @@ void flushcachetodisk(DSTATE *s)
 			}
 		}
 
-		/* update interface timestamp in database */
-		if (!db_setupdated(iterator->interface, iterator->updated)) {
-			handledatabaseerror(s);
-			break;
-		}
-
 		if (!iterator->active && !logcount) {
 			/* throw away if interface hasn't seen any data and is disabled */
 			if (!iterator->currx && !iterator->curtx) {
@@ -650,12 +643,18 @@ void flushcachetodisk(DSTATE *s)
 					break;
 				}
 			}
+		}
 
-			/* update interface activity status in database if changed */
-			if (!db_setactive(iterator->interface, iterator->active)) {
-				handledatabaseerror(s);
-				break;
-			}
+		/* update interface timestamp in database */
+		if (!db_setupdated(iterator->interface, iterator->updated)) {
+			handledatabaseerror(s);
+			break;
+		}
+
+		/* update interface activity status in database */
+		if (!db_setactive(iterator->interface, iterator->active)) {
+			handledatabaseerror(s);
+			break;
 		}
 
 		iterator = iterator->next;
@@ -664,7 +663,6 @@ void flushcachetodisk(DSTATE *s)
 	if (db_intransaction && !db_errcode) {
 		if (!db_committransaction()) {
 			handledatabaseerror(s);
-			db_rollbacktransaction();
 		} else {
 			/* clear xferlog now that everything is in database */
 			iterator = s->dcache;
@@ -677,7 +675,11 @@ void flushcachetodisk(DSTATE *s)
 	} else {
 		db_rollbacktransaction();
 	}
-	timeused(__func__, 0);
+	used_secs = timeused(__func__, 0);
+	if (used_secs > SLOWDBWARNLIMIT) {
+		snprintf(errorstring, 1024, "Writing cached data to database took %.1f seconds.", used_secs);
+		printe(PT_Warning);
+	}
 }
 
 void handledatabaseerror(DSTATE *s)
@@ -687,11 +689,16 @@ void handledatabaseerror(DSTATE *s)
 		printe(PT_Error);
 		errorexitdaemon(s, 1);
 	} else {
-		s->dbretrycount++;
-		if (s->dbretrycount >= DBRETRYLIMIT) {
-			snprintf(errorstring, 1024, "Database error retry limit %d reached, exiting.", DBRETRYLIMIT);
+		if (db_isdiskfull(db_errcode)) {
+			snprintf(errorstring, 1024, "Disk is full, continuing with data caching.");
 			printe(PT_Error);
-			errorexitdaemon(s, 1);
+		} else {
+			s->dbretrycount++;
+			if (s->dbretrycount > DBRETRYLIMIT) {
+				snprintf(errorstring, 1024, "Database error retry limit of %d reached, exiting.", DBRETRYLIMIT);
+				printe(PT_Error);
+				errorexitdaemon(s, 1);
+			}
 		}
 	}
 }
@@ -699,13 +706,13 @@ void handledatabaseerror(DSTATE *s)
 void cleanremovedinterfaces(DSTATE *s)
 {
 	datacache *iterator = s->dcache;
-	dbiflist *dbifl = NULL, *dbifl_iterator = NULL;
+	iflist *dbifl = NULL, *dbifl_iterator = NULL;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	while (iterator != NULL) {
 		if (!db_getinterfacecountbyname(iterator->interface)) {
-			dbiflistadd(&dbifl, iterator->interface);
+			iflistadd(&dbifl, iterator->interface, 0);
 		}
 		iterator = iterator->next;
 	}
@@ -716,13 +723,15 @@ void cleanremovedinterfaces(DSTATE *s)
 			snprintf(errorstring, 1024, "Removing interface \"%s\" from update list.", dbifl_iterator->interface);
 			printe(PT_Info);
 			datacache_remove(&s->dcache, dbifl_iterator->interface);
-			s->dbifcount--;
+			if (s->dbifcount > 0) {
+				s->dbifcount--;
+			}
 			dbifl_iterator = dbifl_iterator->next;
 		}
 		datacache_status(&s->dcache);
-		dbiflistfree(&dbifl);
+		iflistfree(&dbifl);
 	}
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 }
 
 void handleintsignals(DSTATE *s)
@@ -737,9 +746,7 @@ void handleintsignals(DSTATE *s)
 			s->dbifcount = 0;
 			ibwflush();
 			db_close();
-			if (loadcfg(s->cfgfile)) {
-				strncpy_nt(s->dirname, cfg.dbdir, 512);
-			}
+			loadcfg(s->cfgfile);
 			ibwloadcfg(s->cfgfile);
 			if (!db_open_rw(1)) {
 				snprintf(errorstring, 1024, "Opening database after SIGHUP failed (%s), exiting.", strerror(errno));
@@ -772,7 +779,7 @@ void handleintsignals(DSTATE *s)
 			break;
 
 		default:
-			snprintf(errorstring, 1024, "Unkown signal %d received, ignoring.", intsignal);
+			snprintf(errorstring, 1024, "Unknown signal %d received, ignoring.", intsignal);
 			printe(PT_Info);
 			break;
 	}
@@ -783,8 +790,8 @@ void handleintsignals(DSTATE *s)
 void preparedirs(DSTATE *s)
 {
 	/* database directory */
-	if (mkpath(s->dirname, 0775)) {
-		updatedirowner(s->dirname, s->user, s->group);
+	if (mkpath(cfg.dbdir, 0775)) {
+		updatedirowner(cfg.dbdir, s->user, s->group);
 	}
 
 	if (!cfg.createdirs || !s->rundaemon) {
@@ -805,7 +812,7 @@ void datacache_status(datacache **dc)
 	uint32_t bwlimit = 0;
 	datacache *iterator = *dc;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	snprintf(buffer, 1024, "Monitoring (%d): ", datacache_count(dc));
 	b = (unsigned int)strlen(buffer) + 1;
@@ -834,7 +841,7 @@ void datacache_status(datacache **dc)
 		snprintf(errorstring, 1024, "Nothing to monitor");
 	}
 	printe(PT_Info);
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 }
 
 void interfacechangecheck(DSTATE *s)
@@ -844,10 +851,10 @@ void interfacechangecheck(DSTATE *s)
 	uint32_t newhash;
 	int offset, found;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	/* get list of currently visible interfaces */
-	if (getiflist(&ifacelist, 0) == 0) {
+	if (getifliststring(&ifacelist, 0) == 0) {
 		free(ifacelist);
 		s->iflisthash = 0;
 		return;
@@ -908,7 +915,7 @@ void interfacechangecheck(DSTATE *s)
 	free(ifacelist);
 
 	s->iflisthash = newhash;
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 }
 
 uint32_t simplehash(const char *data, int len)
