@@ -1,5 +1,6 @@
 #include "common.h"
 #include "misc.h"
+#include "iflist.h"
 #include "dbsql.h"
 
 /* global db */
@@ -29,6 +30,10 @@ int db_open(const int createifnotfound, const int readonly)
 #else
 	struct stat filestat;
 
+	if (db != NULL) {
+		return 1;
+	}
+
 	snprintf(dbfilename, 530, "%s/%s", cfg.dbdir, DATABASEFILE);
 
 	/* create database if file doesn't exist */
@@ -37,7 +42,7 @@ int db_open(const int createifnotfound, const int readonly)
 			createdb = 1;
 		} else {
 			if (debug)
-				printf("Error: Handling database \"%s\" failed: %s\n", dbfilename, strerror(errno));
+				printf("Error (debug): Handling database \"%s\" failed: %s\n", dbfilename, strerror(errno));
 			return 0;
 		}
 	} else {
@@ -62,11 +67,11 @@ int db_open(const int createifnotfound, const int readonly)
 	if (rc) {
 		db_errcode = rc;
 		if (debug)
-			printf("Error: Can't open database \"%s\": %s\n", dbfilename, sqlite3_errmsg(db));
+			printf("Error (debug): Can't open database \"%s\": %s\n", dbfilename, sqlite3_errmsg(db));
 		return 0;
 	} else {
 		if (debug)
-			printf("Database \"%s\" open\n", dbfilename);
+			printf("Database \"%s\" open (ro: %d)\n", dbfilename, readonly);
 	}
 
 	if (createdb) {
@@ -79,7 +84,7 @@ int db_open(const int createifnotfound, const int readonly)
 #endif
 		if (!db_create()) {
 			if (debug)
-				printf("Error: Creating database \"%s\" structure failed\n", dbfilename);
+				printf("Error (debug): Creating database \"%s\" structure failed\n", dbfilename);
 			db_close();
 			return 0;
 		} else {
@@ -87,7 +92,7 @@ int db_open(const int createifnotfound, const int readonly)
 				printf("Database \"%s\" structure created\n", dbfilename);
 			if (!db_setinfo("dbversion", SQLDBVERSION, 1)) {
 				if (debug)
-					printf("Error: Writing version info to database \"%s\" failed\n", dbfilename);
+					printf("Error (debug): Writing version info to database \"%s\" failed\n", dbfilename);
 				db_close();
 				return 0;
 			}
@@ -95,9 +100,15 @@ int db_open(const int createifnotfound, const int readonly)
 	}
 
 	/* set pragmas */
-	if (!db_setpragmas()) {
-		db_close();
-		return 0;
+	if (!readonly) {
+		sqlite3_busy_timeout(db, cfg.updateinterval * 1000);
+		if (!db_setpragmas()) {
+			db_close();
+			return 0;
+		}
+	} else {
+		/* set busy timeout when database is open in read-only mode */
+		sqlite3_busy_timeout(db, DBREADTIMEOUTSECS * 1000);
 	}
 
 	if (!createdb) {
@@ -160,6 +171,7 @@ int db_validate(const int readonly)
 int db_setpragmas(void)
 {
 	int rc;
+	char sql[25];
 	sqlite3_stmt *sqlstmt;
 
 	/* enable use of foreign keys */
@@ -193,19 +205,64 @@ int db_setpragmas(void)
 		return 0;
 	}
 
+#if HAVE_DECL_SQLITE_CHECKPOINT_RESTART
+	/* set journal_mode */
+	if (cfg.waldb) {
+		if (!db_exec("PRAGMA journal_mode = WAL")) {
+			return 0;
+		}
+	} else {
+		if (!db_exec("PRAGMA journal_mode = DELETE")) {
+			return 0;
+		}
+	}
+#endif
+
+	/* set synchronous */
+	if (cfg.dbsynchronous == -1) {
+#if HAVE_DECL_SQLITE_CHECKPOINT_RESTART
+		if (cfg.waldb) {
+			if (!db_exec("PRAGMA synchronous = 1")) {
+				return 0;
+			}
+		} else {
+			if (!db_exec("PRAGMA synchronous = 2")) {
+				return 0;
+			}
+		}
+#else
+		if (!db_exec("PRAGMA synchronous = 2")) {
+			return 0;
+		}
+#endif
+	} else {
+		snprintf(sql, 25, "PRAGMA synchronous = %d", cfg.dbsynchronous);
+		if (!db_exec(sql)) {
+			return 0;
+		}
+	}
+
 	return 1;
 }
 
 int db_close(void)
 {
 	int rc;
+
+	if (db == NULL) {
+		return 1;
+	}
+
 	rc = sqlite3_close(db);
 	if (rc == SQLITE_OK) {
+		db = NULL;
+		if (debug)
+			printf("Database closed\n");
 		return 1;
 	} else {
 		db_errcode = rc;
 		if (debug)
-			printf("Error: Closing database failed (%d): %s\n", rc, sqlite3_errmsg(db));
+			printf("Error (debug): Closing database failed (%d): %s\n", rc, sqlite3_errmsg(db));
 		return 0;
 	}
 }
@@ -224,7 +281,7 @@ int db_exec(const char *sql)
 	}
 
 	rc = sqlite3_step(sqlstmt);
-	if (rc != SQLITE_DONE) {
+	if (rc != SQLITE_DONE && rc != SQLITE_ROW) {
 		db_errcode = rc;
 		snprintf(errorstring, 1024, "Exec step failed (%d: %s): \"%s\"", rc, sqlite3_errmsg(db), sql);
 		printe(PT_Error);
@@ -335,6 +392,24 @@ int db_removeinterface(const char *iface)
 	return db_exec(sql);
 }
 
+int db_renameinterface(const char *iface, const char *newifname)
+{
+	char sql[128];
+	sqlite3_int64 ifaceid = 0;
+
+	if (!strlen(newifname)) {
+		return 0;
+	}
+
+	ifaceid = db_getinterfaceid(iface, 0);
+	if (ifaceid == 0) {
+		return 0;
+	}
+
+	sqlite3_snprintf(128, sql, "update interface set name='%q' where id=%" PRId64 "", newifname, (int64_t)ifaceid);
+	return db_exec(sql);
+}
+
 uint64_t db_getinterfacecount(void)
 {
 	return db_getinterfacecountbyname("");
@@ -529,7 +604,7 @@ int db_getinterfaceinfo(const char *iface, interfaceinfo *info)
 {
 	int rc;
 	char sql[512], *ifaceidin = NULL;
-	sqlite3_int64 ifaceid = 0;
+	sqlite3_int64 ifaceid;
 	sqlite3_stmt *sqlstmt;
 
 	if (strchr(iface, '+') == NULL) {
@@ -541,6 +616,7 @@ int db_getinterfaceinfo(const char *iface, interfaceinfo *info)
 	} else {
 		ifaceidin = db_getinterfaceidin(iface);
 		if (ifaceidin == NULL || strlen(ifaceidin) < 1) {
+			free(ifaceidin);
 			return 0;
 		}
 		sqlite3_snprintf(512, sql, "select \"%q\", NULL, max(active), max(strftime('%%s', created, 'utc')), min(strftime('%%s', updated, 'utc')), 0, 0, sum(rxtotal), sum(txtotal) from interface where id in (%q)", iface, ifaceidin);
@@ -639,13 +715,22 @@ char *db_getinfo(const char *name)
 	return buffer;
 }
 
-int db_getiflist(dbiflist **dbifl)
+int db_getiflist(iflist **ifl)
+{
+	return db_getiflist_sorted(ifl, 0);
+}
+
+int db_getiflist_sorted(iflist **ifl, const int orderbytraffic)
 {
 	int rc;
 	char *sql;
 	sqlite3_stmt *sqlstmt;
 
-	sql = "select name from interface order by name desc";
+	if (!orderbytraffic) {
+		sql = "select name from interface order by name asc";
+	} else {
+		sql = "select name from interface order by rxtotal+txtotal desc";
+	}
 
 	rc = sqlite3_prepare_v2(db, sql, -1, &sqlstmt, NULL);
 	if (rc != SQLITE_OK) {
@@ -660,7 +745,7 @@ int db_getiflist(dbiflist **dbifl)
 		if (sqlite3_column_text(sqlstmt, 0) == NULL) {
 			continue;
 		}
-		if (!dbiflistadd(dbifl, (const char *)sqlite3_column_text(sqlstmt, 0))) {
+		if (!iflistadd(ifl, (const char *)sqlite3_column_text(sqlstmt, 0), 0)) {
 			break;
 		}
 		rc++;
@@ -741,17 +826,6 @@ int db_addtraffic_dated(const char *iface, const uint64_t rx, const uint64_t tx,
 		}
 	}
 
-	/* change updated only if more recent than previous when timestamp provided */
-	if (timestamp > 0) {
-		sqlite3_snprintf(1024, sql, "update interface set active=1, updated=datetime(%s, 'localtime') where id=%" PRId64 " and updated < datetime(%s, 'localtime')", nowdate, (int64_t)ifaceid, nowdate);
-	} else {
-		sqlite3_snprintf(1024, sql, "update interface set active=1, updated=datetime(%s, 'localtime') where id=%" PRId64 "", nowdate, (int64_t)ifaceid);
-	}
-	if (!db_exec(sql)) {
-		/* no transaction rollback needed here as failure of the first step results in no transaction being active */
-		return 0;
-	}
-
 	/* total */
 	if (rx > 0 || tx > 0) {
 		sqlite3_snprintf(1024, sql, "update interface set rxtotal=rxtotal+%" PRIu64 ", txtotal=txtotal+%" PRIu64 " where id=%" PRId64 "", rx, tx, (int64_t)ifaceid);
@@ -850,11 +924,12 @@ int db_removeoldentries(void)
 		printf("db: removing old entries\n");
 	}
 
-	if (!db_removeoldentries_top()) {
+	if (!db_begintransaction()) {
 		return 0;
 	}
 
-	if (!db_begintransaction()) {
+	if (!db_removeoldentries_top()) {
+		db_rollbacktransaction();
 		return 0;
 	}
 
@@ -920,8 +995,8 @@ int db_removeoldentries_top(void)
 {
 	int errorcount = 0;
 	char sql[512];
-	dbiflist *dbifl = NULL, *dbifl_iterator = NULL;
-	sqlite3_int64 ifaceid = 0;
+	iflist *dbifl = NULL, *dbifl_iterator = NULL;
+	sqlite3_int64 ifaceid;
 
 	if (cfg.topdayentries <= 0) {
 		return 1;
@@ -956,7 +1031,7 @@ int db_removeoldentries_top(void)
 		dbifl_iterator = dbifl_iterator->next;
 	}
 
-	dbiflistfree(&dbifl);
+	iflistfree(&dbifl);
 
 	if (errorcount) {
 		return 0;
@@ -967,6 +1042,9 @@ int db_removeoldentries_top(void)
 
 int db_vacuum(void)
 {
+	if (debug) {
+		printf("db: vacuum\n");
+	}
 	return db_exec("VACUUM");
 }
 
@@ -974,6 +1052,10 @@ int db_begintransaction(void)
 {
 	int rc;
 
+	if (debug) {
+		printf("db: begin transaction\n");
+	}
+
 	rc = sqlite3_exec(db, "BEGIN IMMEDIATE", 0, 0, 0);
 	if (rc) {
 		db_errcode = rc;
@@ -989,15 +1071,23 @@ int db_committransaction(void)
 {
 	int rc;
 
+	if (debug) {
+		printf("db: commit transaction\n");
+	}
+
+	db_intransaction = 0;
+
 	rc = sqlite3_exec(db, "COMMIT", 0, 0, 0);
 	if (rc) {
-		db_errcode = rc;
 		snprintf(errorstring, 1024, "Commit transaction to database failed (%d): %s", rc, sqlite3_errmsg(db));
 		printe(PT_Error);
-		db_intransaction = 0;
+		/* execute rollback if commit failure left the transaction open */
+		if (!sqlite3_get_autocommit(db)) {
+			db_rollbacktransaction();
+		}
+		db_errcode = rc;
 		return 0;
 	}
-	db_intransaction = 0;
 	return 1;
 }
 
@@ -1005,15 +1095,19 @@ int db_rollbacktransaction(void)
 {
 	int rc;
 
+	if (debug) {
+		printf("db: rollback transaction\n");
+	}
+
+	db_intransaction = 0;
+
 	rc = sqlite3_exec(db, "ROLLBACK", 0, 0, 0);
 	if (rc) {
 		db_errcode = rc;
 		snprintf(errorstring, 1024, "Transaction rollback failed (%d): %s", rc, sqlite3_errmsg(db));
 		printe(PT_Error);
-		db_intransaction = 0;
 		return 0;
 	}
-	db_intransaction = 0;
 	return 1;
 }
 
@@ -1031,32 +1125,35 @@ int db_iserrcodefatal(int errcode)
 	}
 }
 
-int dbiflistadd(dbiflist **dbifl, const char *iface)
+int db_isdiskfull(int errcode)
 {
-	dbiflist *newif;
-
-	newif = malloc(sizeof(dbiflist));
-	if (newif == NULL) {
+	if (errcode == SQLITE_FULL) {
+		return 1;
+	} else {
 		return 0;
 	}
-
-	newif->next = *dbifl;
-	*dbifl = newif;
-	strncpy_nt(newif->interface, iface, 32);
-
-	return 1;
 }
 
-void dbiflistfree(dbiflist **dbifl)
+#if HAVE_DECL_SQLITE_CHECKPOINT_RESTART
+void db_walcheckpoint(void)
 {
-	dbiflist *dbifl_prev;
+	double used_secs = 0.0;
 
-	while (*dbifl != NULL) {
-		dbifl_prev = *dbifl;
-		*dbifl = (*dbifl)->next;
-		free(dbifl_prev);
+	timeused(__func__, 1);
+#if HAVE_DECL_SQLITE_CHECKPOINT_TRUNCATE
+	sqlite3_wal_checkpoint_v2(db, NULL, SQLITE_CHECKPOINT_TRUNCATE, NULL, NULL);
+#else
+	sqlite3_wal_checkpoint_v2(db, NULL, SQLITE_CHECKPOINT_RESTART, NULL, NULL);
+#endif
+	timeused(__func__, 0);
+
+	used_secs = timeused(__func__, 0);
+	if (used_secs > SLOWDBWARNLIMIT) {
+		snprintf(errorstring, 1024, "Write-Ahead Logging checkpoint took %.1f seconds.", used_secs);
+		printe(PT_Warning);
 	}
 }
+#endif
 
 int db_getdata(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *iface, const char *table, const uint32_t resultlimit)
 {
@@ -1075,11 +1172,6 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 
 	listinfo->count = 0;
 
-	ifaceidin = db_getinterfaceidin(iface);
-	if (ifaceidin == NULL) {
-		return 0;
-	}
-
 	ret = 0;
 	for (i = 0; i < 6; i++) {
 		if (strcmp(table, datatables[i]) == 0) {
@@ -1091,6 +1183,11 @@ int db_getdata_range(dbdatalist **dbdata, dbdatalistinfo *listinfo, const char *
 		return 0;
 	}
 
+	ifaceidin = db_getinterfaceidin(iface);
+	if (ifaceidin == NULL) {
+		return 0;
+	}
+
 	dbegin[0] = '\0';
 	if (strlen(databegin)) {
 		snprintf(dbegin, 32, "and date >= '%s'", databegin);
@@ -1250,7 +1347,7 @@ unsigned int getqueryinterfacecount(const char *input)
 {
 	unsigned int i, ifacecount = 1;
 
-	if (input[0] == '+' || input[strlen(input) - 1] == '+' || !strlen(input)) {
+	if (!strlen(input) || input[0] == '+' || input[strlen(input) - 1] == '+') {
 		return 0;
 	}
 
