@@ -1,9 +1,4 @@
 #include "common.h"
-#include "misc.h"
-#include "dbaccess.h"
-#include "dbsql.h"
-#include "cfg.h"
-#include "ibw.h"
 #include "ifinfo.h"
 
 int getifinfo(const char *iface)
@@ -53,7 +48,7 @@ int getifinfo(const char *iface)
 		return 1;
 	}
 
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
+#elif defined(BSD_VNSTAT)
 	if (readifaddrs(inface) == 1) {
 		ifinfo.timestamp = time(NULL);
 		return 1;
@@ -65,28 +60,69 @@ int getifinfo(const char *iface)
 	return 0;
 }
 
-int getiflist(char **ifacelist, int showspeed)
+int getifliststring(char **ifacelist, int showspeed)
 {
-	uint32_t speed;
 	char temp[64];
-#if defined(__linux__) || defined(CHECK_VNSTAT)
-	char interface[32];
-	FILE *fp;
-	DIR *dp;
-	struct dirent *di;
-	char procline[512];
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
-	struct ifaddrs *ifap, *ifa;
-#endif
+	iflist *ifl = NULL, *ifl_iterator = NULL;
 
-	/* initialize list */
+	/* initialize list string */
 	*ifacelist = (char *)malloc(sizeof(char));
 	if (*ifacelist == NULL) {
 		panicexit(__FILE__, __LINE__);
 	}
 	*ifacelist[0] = '\0';
 
+	if (getiflist(&ifl, showspeed, 1) > 0) {
+
+		ifl_iterator = ifl;
+
+		while (ifl_iterator != NULL) {
+			*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(ifl_iterator->interface) + 2) * sizeof(char)));
+			if (*ifacelist == NULL) {
+				panicexit(__FILE__, __LINE__);
+			}
+			strcat(*ifacelist, ifl_iterator->interface);
+			strcat(*ifacelist, " ");
+
+			if (showspeed && ifl_iterator->bandwidth > 0) {
+				snprintf(temp, 64, "(%u Mbit) ", ifl_iterator->bandwidth);
+				*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(temp) + 1) * sizeof(char)));
+				if (*ifacelist == NULL) {
+					panicexit(__FILE__, __LINE__);
+				}
+				strcat(*ifacelist, temp);
+			}
+
+			ifl_iterator = ifl_iterator->next;
+		}
+
+		iflistfree(&ifl);
+		return 1;
+	}
+
+	iflistfree(&ifl);
+	return 0;
+}
+
+int getiflist(iflist **ifl, const int getspeed, const int validate)
+{
 #if defined(__linux__) || defined(CHECK_VNSTAT)
+	return getiflist_linux(ifl, getspeed, validate);
+#elif defined(BSD_VNSTAT)
+	return getiflist_bsd(ifl, getspeed, validate);
+#endif
+}
+
+#if defined(__linux__) || defined(CHECK_VNSTAT)
+int getiflist_linux(iflist **ifl, const int getspeed, const int validate)
+{
+	char temp[64];
+	char interface[32];
+	FILE *fp;
+	DIR *dp;
+	struct dirent *di;
+	char procline[512];
+
 	if ((fp = fopen(PROCNETDEV, "r")) != NULL) {
 
 		/* make list of interfaces */
@@ -94,23 +130,13 @@ int getiflist(char **ifacelist, int showspeed)
 			sscanf(procline, "%63s", temp);
 			if (strlen(temp) > 0 && (isdigit(temp[(strlen(temp) - 1)]) || temp[(strlen(temp) - 1)] == ':')) {
 				sscanf(temp, "%31[^':']s", interface);
-				*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(interface) + 2) * sizeof(char)));
-				if (*ifacelist == NULL) {
-					panicexit(__FILE__, __LINE__);
-				}
-				strcat(*ifacelist, interface);
-				strcat(*ifacelist, " ");
-				if (!showspeed) {
+				if (validate && !isifvalid(interface)) {
 					continue;
 				}
-				speed = getifspeed(interface);
-				if (speed > 0) {
-					snprintf(temp, 64, "(%u Mbit) ", speed);
-					*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(temp) + 1) * sizeof(char)));
-					if (*ifacelist == NULL) {
-						panicexit(__FILE__, __LINE__);
-					}
-					strcat(*ifacelist, temp);
+				if (getspeed) {
+					iflistadd(ifl, interface, getifspeed(interface));
+				} else {
+					iflistadd(ifl, interface, 0);
 				}
 			}
 		}
@@ -127,23 +153,13 @@ int getiflist(char **ifacelist, int showspeed)
 				if (di->d_name[0] == '.' || strlen(di->d_name) > 31) {
 					continue;
 				}
-				*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(di->d_name) + 2) * sizeof(char)));
-				if (*ifacelist == NULL) {
-					panicexit(__FILE__, __LINE__);
-				}
-				strcat(*ifacelist, di->d_name);
-				strcat(*ifacelist, " ");
-				if (!showspeed) {
+				if (validate && !isifvalid(di->d_name)) {
 					continue;
 				}
-				speed = getifspeed(di->d_name);
-				if (speed > 0) {
-					snprintf(temp, 64, "(%u Mbit) ", speed);
-					*ifacelist = (char *)realloc(*ifacelist, ((strlen(*ifacelist) + strlen(temp) + 1) * sizeof(char)));
-					if (*ifacelist == NULL) {
-						panicexit(__FILE__, __LINE__);
-					}
-					strcat(*ifacelist, temp);
+				if (getspeed) {
+					iflistadd(ifl, di->d_name, getifspeed(di->d_name));
+				} else {
+					iflistadd(ifl, di->d_name, 0);
 				}
 			}
 
@@ -152,7 +168,13 @@ int getiflist(char **ifacelist, int showspeed)
 		}
 	}
 
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
+	return 0;
+}
+#elif defined(BSD_VNSTAT)
+int getiflist_bsd(iflist **ifl, const int getspeed, const int validate)
+{
+	struct ifaddrs *ifap, *ifa;
+
 	if (getifaddrs(&ifap) >= 0) {
 
 		/* make list of interfaces */
@@ -160,23 +182,13 @@ int getiflist(char **ifacelist, int showspeed)
 			if (ifa->ifa_addr->sa_family != AF_LINK || strlen(ifa->ifa_name) > 31) {
 				continue;
 			}
-			*ifacelist = realloc(*ifacelist, ((strlen(*ifacelist) + strlen(ifa->ifa_name) + 2) * sizeof(char)));
-			if (*ifacelist == NULL) {
-				panicexit(__FILE__, __LINE__);
-			}
-			strcat(*ifacelist, ifa->ifa_name);
-			strcat(*ifacelist, " ");
-			if (!showspeed) {
+			if (validate && !isifvalid(ifa->ifa_name)) {
 				continue;
 			}
-			speed = getifspeed(ifa->ifa_name);
-			if (speed > 0) {
-				snprintf(temp, 64, "(%u Mbit) ", speed);
-				*ifacelist = realloc(*ifacelist, ((strlen(*ifacelist) + strlen(temp) + 1) * sizeof(char)));
-				if (*ifacelist == NULL) {
-					panicexit(__FILE__, __LINE__);
-				}
-				strcat(*ifacelist, temp);
+			if (getspeed) {
+				iflistadd(ifl, ifa->ifa_name, getifspeed(ifa->ifa_name));
+			} else {
+				iflistadd(ifl, ifa->ifa_name, 0);
 			}
 		}
 
@@ -184,10 +196,9 @@ int getiflist(char **ifacelist, int showspeed)
 		return 1;
 	}
 
-#endif
-
 	return 0;
 }
+#endif
 
 int readproc(const char *iface)
 {
@@ -197,7 +208,7 @@ int readproc(const char *iface)
 
 	if ((fp = fopen(PROCNETDEV, "r")) == NULL) {
 		if (debug)
-			printf("Error: Unable to read %s: %s\n", PROCNETDEV, strerror(errno));
+			printf("Error (debug): Unable to read %s: %s\n", PROCNETDEV, strerror(errno));
 		return 0;
 	}
 
@@ -328,7 +339,7 @@ int readsysclassnet(const char *iface)
 	return 1;
 }
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
+#if defined(BSD_VNSTAT)
 int getifdata(const char *iface, struct if_data *ifd)
 {
 	struct ifaddrs *ifap, *ifa;
@@ -385,6 +396,7 @@ int readifaddrs(const char *iface)
 uint32_t getifspeed(const char *iface)
 {
 	uint64_t speed = 0;
+
 #if defined(__linux__)
 
 	FILE *fp;
@@ -408,7 +420,7 @@ uint32_t getifspeed(const char *iface)
 	}
 	fclose(fp);
 
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
+#elif defined(BSD_VNSTAT)
 
 	struct if_data ifd;
 
@@ -421,6 +433,7 @@ uint32_t getifspeed(const char *iface)
 	}
 
 #endif
+
 	if (debug)
 		printf("getifspeed: \"%s\": %" PRIu64 "\n", iface, speed);
 
@@ -441,3 +454,17 @@ int isifavailable(const char *iface)
 
 	return ret;
 }
+
+int isifvalid(const char *iface)
+{
+	if (strstr(iface, ":") != NULL) {
+		return 0;
+	} else if (strcmp(iface, "lo") == 0) {
+		return 0;
+	} else if (strcmp(iface, "lo0") == 0) {
+		return 0;
+	} else if (strcmp(iface, "sit0") == 0) {
+		return 0;
+	}
+	return 1;
+}
