@@ -34,10 +34,10 @@ void drawimage(IMAGECONTENT *ic)
 			drawsummary(ic, 0, 0);
 			break;
 		case 51:
-			drawsummary(ic, 1, cfg.hourlyrate);
+			drawsummary(ic, 1, cfg.hourlyrate); // horizontal
 			break;
 		case 52:
-			drawsummary(ic, 2, cfg.hourlyrate);
+			drawsummary(ic, 2, cfg.hourlyrate); // vertical
 			break;
 		case 7:
 			drawhourly(ic, cfg.hourlyrate);
@@ -264,7 +264,7 @@ void drawdonut(IMAGECONTENT *ic, const int x, const int y, const float rxp, cons
 	if (rxarc) {
 		gdImageFilledArc(ic->im, x, y, DOUTRAD, DOUTRAD, 270 + txarc, 270 + txarc + rxarc, ic->crxd, gdEdged | gdNoFill);
 		if (rxarc >= 5) {
-			gdImageFill(ic->im, (int)(x + (DOUTRAD / 2 - 3) * cos(((270 * 2 + 2 * txarc + rxarc) / 2) * M_PI / 180)), (int)(y + (DOUTRAD / 2 - 3) * sin(((270 * 2 + 2 * txarc + rxarc) / 2) * M_PI / 180)), ic->crx);
+			gdImageFill(ic->im, (int)(x + (DOUTRAD / 2 - 3) * cos((int)((270 * 2 + 2 * txarc + rxarc) / 2) * M_PI / 180)), (int)(y + (DOUTRAD / 2 - 3) * sin((int)((270 * 2 + 2 * txarc + rxarc) / 2) * M_PI / 180)), ic->crx);
 		}
 		gdImageFilledArc(ic->im, x, y, DINRAD, DINRAD, 270 + txarc, 270 + txarc + rxarc, ic->crxd, gdEdged | gdNoFill);
 	}
@@ -273,6 +273,7 @@ void drawdonut(IMAGECONTENT *ic, const int x, const int y, const float rxp, cons
 	gdImageFilledArc(ic->im, x, y, DINRAD - 2, DINRAD - 2, 0, 360, ic->cbackground, 0);
 }
 
+#ifdef CHECK_VNSTAT
 void drawdonut_libgd_native(IMAGECONTENT *ic, const int x, const int y, const float rxp, const float txp)
 {
 	int rxarc = 0, txarc = 0;
@@ -304,8 +305,9 @@ void drawdonut_libgd_native(IMAGECONTENT *ic, const int x, const int y, const fl
 	// remove center from background filled circle, making it a donut
 	gdImageFilledArc(ic->im, x, y, DINRAD - 2, DINRAD - 2, 0, 360, ic->cbackground, 0);
 }
+#endif
 
-void drawhours(IMAGECONTENT *ic, const int x, const int y, const int rate)
+int drawhours(IMAGECONTENT *ic, const int x, const int y, const int rate)
 {
 	int i, tmax = 0, s = 0, step, prev = 0, diff = 0, chour;
 	float ratediv;
@@ -323,11 +325,12 @@ void drawhours(IMAGECONTENT *ic, const int x, const int y, const int rate)
 
 	if (!db_getdata(&datalist, &datainfo, ic->interface.name, "hour", 24)) {
 		printf("Error: Failed to fetch hour data.\n");
-		return;
+		return 0;
 	}
 
 	if (datainfo.count == 0) {
-		return;
+		gdImageString(ic->im, gdFontGetSmall(), x + 188, y + 54, (unsigned char *)"no data available", ic->ctext);
+		return 0;
 	}
 
 	datalist_i = datalist;
@@ -443,6 +446,8 @@ void drawhours(IMAGECONTENT *ic, const int x, const int y, const int rate)
 	gdImageLine(ic->im, x + 36, y - 9, x + 38, y - 6, ic->ctext);
 	gdImageLine(ic->im, x + 36, y - 9, x + 34, y - 6, ic->ctext);
 	gdImageLine(ic->im, x + 34, y - 6, x + 38, y - 6, ic->ctext);
+
+	return 1;
 }
 
 void drawhourly(IMAGECONTENT *ic, const int rate)
@@ -475,16 +480,18 @@ void drawhourly(IMAGECONTENT *ic, const int rate)
 	}
 
 	layoutinit(ic, buffer, width, height);
-	drawlegend(ic, 242, 183 - headermod);
-	drawhours(ic, 12, 46 - headermod, rate);
+	if (drawhours(ic, 12, 46 - headermod, rate)) {
+		drawlegend(ic, 242, 183 - headermod);
+	}
 }
 
 void drawlist(IMAGECONTENT *ic, const char *listname)
 {
-	int listtype, textx, texty, offsetx = 0, offsety = 0;
+	ListType listtype = LT_None;
+	int textx, texty, offsetx = 0, offsety = 0;
 	int width, height, headermod, i = 1, rowcount = 0;
 	int32_t limit;
-	uint64_t e_rx, e_tx, e_secs = 86400, div, mult;
+	uint64_t e_rx, e_tx, e_secs, div, mult;
 	char buffer[512], datebuff[16], daybuff[16];
 	char stampformat[64], titlename[16], colname[8];
 	struct tm *d;
@@ -493,37 +500,37 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	dbdatalistinfo datainfo;
 
 	if (strcmp(listname, "day") == 0) {
-		listtype = 1;
+		listtype = LT_Day;
 		strncpy_nt(colname, listname, 8);
 		snprintf(titlename, 16, "daily");
 		strncpy_nt(stampformat, cfg.dformat, 64);
 		limit = cfg.listdays;
 	} else if (strcmp(listname, "month") == 0) {
-		listtype = 2;
+		listtype = LT_Month;
 		strncpy_nt(colname, listname, 8);
 		snprintf(titlename, 16, "monthly");
 		strncpy_nt(stampformat, cfg.mformat, 64);
 		limit = cfg.listmonths;
 	} else if (strcmp(listname, "year") == 0) {
-		listtype = 3;
+		listtype = LT_Year;
 		strncpy_nt(colname, listname, 8);
 		snprintf(titlename, 16, "yearly");
 		strncpy_nt(stampformat, "%Y", 64);
 		limit = cfg.listyears;
 	} else if (strcmp(listname, "top") == 0) {
-		listtype = 4;
+		listtype = LT_Top;
 		snprintf(colname, 8, "day");
 		strncpy_nt(stampformat, cfg.tformat, 64);
 		limit = cfg.listtop;
 		offsetx = 30;
 	} else if (strcmp(listname, "hour") == 0) {
-		listtype = 5;
+		listtype = LT_Hour;
 		strncpy_nt(colname, listname, 8);
 		snprintf(titlename, 16, "hourly");
 		strncpy_nt(stampformat, "%H:%M", 64);
 		limit = cfg.listhours;
 	} else if (strcmp(listname, "fiveminute") == 0) {
-		listtype = 6;
+		listtype = LT_5min;
 		strncpy_nt(colname, "time", 8);
 		snprintf(titlename, 16, "5 minute");
 		strncpy_nt(stampformat, "%H:%M", 64);
@@ -537,7 +544,6 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	}
 
 	daybuff[0] = '\0';
-	e_rx = e_tx = 0;
 
 	if (!db_getdata_range(&datalist, &datainfo, ic->interface.name, listname, (uint32_t)limit, ic->databegin, ic->dataend)) {
 		printf("Error: Failed to fetch %s data.\n", "day");
@@ -546,7 +552,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 
 	datalist_i = datalist;
 
-	if (listtype == 4) {
+	if (listtype == LT_Top) {
 		if (limit > 0 && datainfo.count < (uint32_t)limit) {
 			limit = (int32_t)datainfo.count;
 		}
@@ -557,7 +563,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 		}
 	}
 
-	if (listtype == 5 || listtype == 6) {
+	if (listtype == LT_Hour || listtype == LT_5min) {
 		while (datalist_i != NULL) {
 			d = localtime(&datalist_i->timestamp);
 			strftime(datebuff, 16, cfg.dformat, d);
@@ -573,14 +579,23 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	rowcount += datainfo.count;
 
 	width = 500;
-	if (listtype >= 4 && (datainfo.count < 2 || strlen(ic->dataend) == 0 || listtype == 4)) { // less space needed when no estimate or sum is shown
+	height = 98;
+
+	// less space needed when no estimate or sum is shown (Top, 5min and Hour never have estimate)
+	if ((listtype == LT_Day || listtype == LT_Month || listtype == LT_Year || listtype == LT_Top || listtype == LT_5min || listtype == LT_Hour) &&
+		(datainfo.count < 2 || listtype == LT_Top || listtype == LT_5min || listtype == LT_Hour)) {
 		height = 86;
 		offsety = -16;
-	} else {
-		height = 98;
 	}
+	// exception for 5min and Hour when having sum shown
+	if ((listtype == LT_5min || listtype == LT_Hour) && datainfo.count > 1 && strlen(ic->dataend) > 0) {
+		height = 98;
+		offsety = 0;
+	}
+
 	height += 12 * rowcount;
 
+	// "no data available"
 	if (!datainfo.count) {
 		height = 98;
 		offsety = -24;
@@ -610,7 +625,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	layoutinit(ic, buffer, width, height);
 
 	if (datainfo.count) {
-		if (listtype == 4) { // top
+		if (listtype == LT_Top) {
 			if (cfg.ostyle <= 2) {
 				drawlegend(ic, 398, 40 - headermod);
 			}
@@ -629,7 +644,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	textx = 10;
 	texty = 40 - headermod;
 
-	if (listtype == 4) { // top
+	if (listtype == LT_Top) { // top
 		snprintf(buffer, 512, "   #      day        rx           tx          total");
 	} else { // everything else
 		snprintf(buffer, 512, " %8s       rx           tx          total", colname);
@@ -651,7 +666,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	while (datalist_i != NULL) {
 		d = localtime(&datalist_i->timestamp);
 
-		if (listtype == 5 || listtype == 6) {
+		if (listtype == LT_5min || listtype == LT_Hour) {
 			strftime(datebuff, 16, cfg.dformat, d);
 			if (strcmp(daybuff, datebuff) != 0) {
 				snprintf(buffer, 32, " %s", datebuff);
@@ -661,7 +676,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 			}
 		}
 
-		if (listtype == 4) {
+		if (listtype == LT_Top) {
 			if (strftime(datebuff, 16, stampformat, d) <= 8) {
 				snprintf(buffer, 32, "  %2d   %*s", i, getpadding(8, datebuff), datebuff);
 				strcat(buffer, "   ");
@@ -690,42 +705,39 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 		strncat(buffer, getvalue(datalist_i->rx + datalist_i->tx, 10, RT_Normal), 32);
 		if (cfg.ostyle > 2) {
 			strcat(buffer, "  ");
-			if (datalist_i->next == NULL) {
+			e_secs = 0;
+			if (datalist_i->next == NULL && issametimeslot(listtype, datalist_i->timestamp, ic->interface.updated)) {
 				d = localtime(&ic->interface.updated);
-				if (listtype == 1) { // day
+				if (listtype == LT_Day) {
 					e_secs = (uint64_t)(d->tm_sec + (d->tm_min * 60) + (d->tm_hour * 3600));
-				} else if (listtype == 2) { // month
+				} else if (listtype == LT_Month) {
 					e_secs = (uint64_t)mosecs(datalist_i->timestamp, ic->interface.updated);
-				} else if (listtype == 3) { // year
+				} else if (listtype == LT_Year) {
 					e_secs = (uint64_t)(d->tm_yday * 86400 + d->tm_sec + (d->tm_min * 60) + (d->tm_hour * 3600));
-				} else if (listtype == 4) { // top
+				} else if (listtype == LT_Top) {
 					e_secs = 86400;
-				} else if (listtype == 5) { // hour
+				} else if (listtype == LT_Hour) {
 					e_secs = (uint64_t)(d->tm_sec + (d->tm_min * 60));
-				} else if (listtype == 6) { // 5min
-					if ((ic->interface.updated - (ic->interface.updated % 300)) == (datalist_i->timestamp - (datalist_i->timestamp % 300))) {
-						e_secs = (uint64_t)(d->tm_sec + (d->tm_min % 5 * 60));
-					} else {
-						e_secs = 300;
-					}
+				} else if (listtype == LT_5min) {
+					e_secs = (uint64_t)(d->tm_sec + (d->tm_min % 5 * 60));
 				}
 			} else {
-				if (listtype == 1 || listtype == 4) { // day
+				if (listtype == LT_Day || listtype == LT_Top) {
 					e_secs = 86400;
-				} else if (listtype == 2) { // month
+				} else if (listtype == LT_Month) {
 					e_secs = (uint64_t)(dmonth(d->tm_mon) * 86400);
-				} else if (listtype == 3) { // year
+				} else if (listtype == LT_Year) {
 					e_secs = (uint64_t)((365 + isleapyear(d->tm_year + 1900)) * 86400);
-				} else if (listtype == 5) { // hour
+				} else if (listtype == LT_Hour) {
 					e_secs = 3600;
-				} else if (listtype == 6) { // 5min
+				} else if (listtype == LT_5min) {
 					e_secs = 300;
 				}
 			}
 			strncat(buffer, gettrafficrate(datalist_i->rx + datalist_i->tx, (time_t)e_secs, 14), 32);
 		}
 		gdImageString(ic->im, gdFontGetSmall(), textx, texty, (unsigned char *)buffer, ic->ctext);
-		if (listtype == 4) { // top
+		if (listtype == LT_Top) {
 			if (cfg.ostyle > 2) {
 				drawbar(ic, textx + 428, texty + 4, 52, datalist_i->rx, datalist_i->tx, datainfo.max);
 			} else {
@@ -761,7 +773,7 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 		gdImageLine(ic->im, textx + 2, texty + 5, textx + 296 + offsetx, texty + 5, ic->cline);
 	}
 
-	if ((strlen(ic->dataend) == 0 && datainfo.count > 0 && listtype < 4) || (strlen(ic->dataend) > 0 && datainfo.count > 1 && listtype != 4)) {
+	if ((strlen(ic->dataend) == 0 && datainfo.count > 0 && (listtype == LT_Day || listtype == LT_Month || listtype == LT_Year)) || (strlen(ic->dataend) > 0 && datainfo.count > 1 && listtype != LT_Top)) {
 
 		d = localtime(&ic->interface.updated);
 		if (datalist_i->rx == 0 || datalist_i->tx == 0 || strlen(ic->dataend) > 0) {
@@ -769,13 +781,13 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 		} else {
 			div = 0;
 			mult = 0;
-			if (listtype == 1) { // day
+			if (listtype == LT_Day) {
 				div = (uint64_t)(d->tm_hour * 60 + d->tm_min);
 				mult = 1440;
-			} else if (listtype == 2) { // month
+			} else if (listtype == LT_Month) {
 				div = (uint64_t)mosecs(datalist_i->timestamp, ic->interface.updated);
 				mult = (uint64_t)(dmonth(d->tm_mon) * 86400);
-			} else if (listtype == 3) { // year
+			} else if (listtype == LT_Year) {
 				div = (uint64_t)(d->tm_yday * 1440 + d->tm_hour * 60 + d->tm_min);
 				mult = (uint64_t)(1440 * (365 + isleapyear(d->tm_year + 1900)));
 			}
@@ -813,27 +825,23 @@ void drawlist(IMAGECONTENT *ic, const char *listname)
 	dbdatalistfree(&datalist);
 }
 
-void drawsummary(IMAGECONTENT *ic, int type, int rate)
+void drawsummary(IMAGECONTENT *ic, const int layout, const int rate)
 {
-	int textx, texty, offset = 0;
 	int width, height, headermod;
-	float rxp = 50, txp = 50, mod;
-	char buffer[512], datebuff[16], daytemp[32];
-	time_t yesterday;
-	struct tm *d;
-	dbdatalist *datalist = NULL;
-	dbdatalist *data_current = NULL, *data_previous = NULL;
-	dbdatalistinfo datainfo;
+	char buffer[512];
 
-	switch (type) {
+	switch (layout) {
+		// horizontal
 		case 1:
 			width = 980;
 			height = 200;
 			break;
+		// vertical
 		case 2:
 			width = 500;
 			height = 370;
 			break;
+		// no hours
 		default:
 			width = 500;
 			height = 200;
@@ -847,8 +855,6 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 		headermod = 0;
 	}
 
-	yesterday = ic->current - 86400;
-
 	ic->im = gdImageCreate(width, height);
 
 	colorinit(ic);
@@ -865,19 +871,92 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 
 	layoutinit(ic, buffer, width, height);
 
-	if (!db_getdata(&datalist, &datainfo, ic->interface.name, "day", 2)) {
-		printf("Error: Failed to fetch day data.\n");
+	if (ic->interface.rxtotal == 0 && ic->interface.txtotal == 0) {
+		gdImageString(ic->im, gdFontGetSmall(), 200, 100, (unsigned char *)"no data available", ic->ctext);
 		return;
 	}
 
-	if (datalist->next == NULL) {
+	drawsummary_alltime(ic, 385, 57 - headermod);
+	drawlegend(ic, 410, 155 - headermod);
+
+	drawsummary_digest(ic, 100, 30 - headermod, "day");
+	drawsummary_digest(ic, 100, 113 - headermod, "month");
+
+	switch (layout) {
+		// horizontal
+		case 1:
+			drawhours(ic, 500, 46 - headermod, rate);
+			break;
+		// vertical
+		case 2:
+			drawhours(ic, 12, 215 - headermod, rate);
+			break;
+		default:
+			break;
+	}
+}
+
+void drawsummary_alltime(IMAGECONTENT *ic, const int x, const int y)
+{
+	struct tm *d;
+	char buffer[512], datebuff[16], daytemp[32];
+
+	gdImageString(ic->im, gdFontGetLarge(), x + 12, y, (unsigned char *)"all time", ic->ctext);
+	snprintf(buffer, 4, "rx ");
+	strncat(buffer, getvalue(ic->interface.rxtotal, 12, RT_Normal), 32);
+	gdImageString(ic->im, gdFontGetSmall(), x, y + 24, (unsigned char *)buffer, ic->ctext);
+	snprintf(buffer, 4, "tx ");
+	strncat(buffer, getvalue(ic->interface.txtotal, 12, RT_Normal), 32);
+	gdImageString(ic->im, gdFontGetSmall(), x, y + 36, (unsigned char *)buffer, ic->ctext);
+	snprintf(buffer, 4, " = ");
+	strncat(buffer, getvalue(ic->interface.rxtotal + ic->interface.txtotal, 12, RT_Normal), 32);
+	gdImageString(ic->im, gdFontGetSmall(), x, y + 50, (unsigned char *)buffer, ic->ctext);
+	d = localtime(&ic->interface.created);
+	strftime(datebuff, 16, cfg.tformat, d);
+	snprintf(daytemp, 24, "since %s", datebuff);
+	snprintf(buffer, 32, "%23s", daytemp);
+	gdImageString(ic->im, gdFontGetSmall(), x - 48, y + 70, (unsigned char *)buffer, ic->ctext);
+}
+
+void drawsummary_digest(IMAGECONTENT *ic, const int x, const int y, const char *mode)
+{
+	int textx, texty, offset = 0;
+	float rxp = 50, txp = 50, mod;
+	char buffer[512], datebuff[16], daytemp[32];
+	time_t yesterday;
+	struct tm *d = NULL;
+	dbdatalist *datalist = NULL;
+	dbdatalist *data_current = NULL, *data_previous = NULL;
+	dbdatalistinfo datainfo;
+
+	yesterday = ic->current - 86400;
+
+	switch(mode[0]) {
+		case 'd':
+			break;
+		case 'm':
+			break;
+		default:
+			printf("Error: Unsupported mode %s for summary digest\n", mode);
+			return;
+	}
+
+	if (!db_getdata(&datalist, &datainfo, ic->interface.name, mode, 2)) {
+		printf("Error: Failed to fetch %s data.\n", mode);
+		return;
+	}
+
+	if (datalist == NULL) {
+		gdImageString(ic->im, gdFontGetSmall(), 150, y + 30, (unsigned char *)"no data available", ic->ctext);
+		return;
+	} else if (datalist->next == NULL) {
 		data_current = datalist;
 	} else {
 		data_previous = datalist;
 		data_current = datalist->next;
 	}
 
-	/* today / latest day */
+	/* latest entry */
 	if (data_current->rx + data_current->tx == 0) {
 		rxp = txp = 0;
 	} else {
@@ -897,22 +976,27 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 		offset = 85;
 	}
 
-	drawdonut(ic, 150 + offset, 75 - headermod, rxp, txp);
+	textx = x + offset;
+	texty = y;
 
-	textx = 100 + offset;
-	texty = 30 - headermod;
+	drawdonut(ic, textx + 50, texty + 45, rxp, txp);
 
-	/* get formated date for today */
-	d = localtime(&ic->current);
-	strftime(datebuff, 16, cfg.dformat, d);
+	if (mode[0] == 'd') {
+		/* get formated date for today */
+		d = localtime(&ic->current);
+		strftime(datebuff, 16, cfg.dformat, d);
 
-	/* get formated date for current day in database */
-	d = localtime(&data_current->timestamp);
-	strftime(daytemp, 16, cfg.dformat, d);
+		/* get formated date for current day in database */
+		d = localtime(&data_current->timestamp);
+		strftime(daytemp, 16, cfg.dformat, d);
 
-	/* change daytemp to today if formated days match */
-	if (strcmp(datebuff, daytemp) == 0) {
-		strncpy_nt(daytemp, "today", 32);
+		/* change daytemp to today if formated days match */
+		if (strcmp(datebuff, daytemp) == 0) {
+			strncpy_nt(daytemp, "today", 32);
+		}
+	} else if (mode[0] == 'm') {
+		d = localtime(&data_current->timestamp);
+		strftime(daytemp, 16, cfg.mformat, d);
 	}
 
 	snprintf(buffer, 32, "%*s", getpadding(12, daytemp), daytemp);
@@ -920,7 +1004,11 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 
 	if (cfg.summaryrate) {
 		d = localtime(&ic->interface.updated);
-		snprintf(buffer, 16, "%15s", gettrafficrate(data_current->rx + data_current->tx, d->tm_sec + (d->tm_min * 60) + (d->tm_hour * 3600), 15));
+		if (mode[0] == 'd') {
+			snprintf(buffer, 16, "%15s", gettrafficrate(data_current->rx + data_current->tx, d->tm_sec + (d->tm_min * 60) + (d->tm_hour * 3600), 15));
+		} else if (mode[0] == 'm') {
+			snprintf(buffer, 16, "%15s", gettrafficrate(data_current->rx + data_current->tx, mosecs(data_current->timestamp, ic->interface.updated), 15));
+		}
 		gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 58, (unsigned char *)buffer, ic->ctext);
 	} else {
 		texty += 7;
@@ -936,7 +1024,7 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 	strncat(buffer, getvalue(data_current->rx + data_current->tx, 12, RT_Normal), 32);
 	gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 44, (unsigned char *)buffer, ic->ctext);
 
-	/* yesterday */
+	/* previous entry */
 	if (data_previous != NULL) {
 		if (data_previous->rx + data_previous->tx == 0) {
 			rxp = txp = 0;
@@ -952,29 +1040,37 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 			txp = txp * mod;
 		}
 
-		drawdonut(ic, 330, 75 - headermod, rxp, txp);
+		textx += 180;
 
-		textx = 280;
-		texty = 30 - headermod;
+		drawdonut(ic, textx + 50, texty + 45, rxp, txp);
 
-		/* get formated date for yesterday */
-		d = localtime(&yesterday);
-		strftime(datebuff, 16, cfg.dformat, d);
+		if (mode[0] == 'd') {
+			/* get formated date for yesterday */
+			d = localtime(&yesterday);
+			strftime(datebuff, 16, cfg.dformat, d);
 
-		/* get formated date for previous day in database */
-		d = localtime(&data_previous->timestamp);
-		strftime(daytemp, 16, cfg.dformat, d);
+			/* get formated date for previous day in database */
+			d = localtime(&data_previous->timestamp);
+			strftime(daytemp, 16, cfg.dformat, d);
 
-		/* change daytemp to yesterday if formated days match */
-		if (strcmp(datebuff, daytemp) == 0) {
-			strncpy_nt(daytemp, "yesterday", 32);
+			/* change daytemp to yesterday if formated days match */
+			if (strcmp(datebuff, daytemp) == 0) {
+				strncpy_nt(daytemp, "yesterday", 32);
+			}
+		} else if (mode[0] == 'm') {
+			d = localtime(&data_previous->timestamp);
+			strftime(daytemp, 16, cfg.mformat, d);
 		}
 
 		snprintf(buffer, 32, "%*s", getpadding(12, daytemp), daytemp);
 		gdImageString(ic->im, gdFontGetLarge(), textx - 54, texty - 1, (unsigned char *)buffer, ic->ctext);
 
 		if (cfg.summaryrate) {
-			snprintf(buffer, 16, "%15s", gettrafficrate(data_previous->rx + data_previous->tx, 86400, 15));
+			if (mode[0] == 'd') {
+				snprintf(buffer, 16, "%15s", gettrafficrate(data_previous->rx + data_previous->tx, 86400, 15));
+			} else if (mode[0] == 'm') {
+				snprintf(buffer, 16, "%15s", gettrafficrate(data_previous->rx + data_previous->tx, dmonth(d->tm_mon) * 86400, 15));
+			}
 			gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 58, (unsigned char *)buffer, ic->ctext);
 		} else {
 			texty += 7;
@@ -994,151 +1090,6 @@ void drawsummary(IMAGECONTENT *ic, int type, int rate)
 	data_current = NULL;
 	data_previous = NULL;
 	dbdatalistfree(&datalist);
-
-	if (!db_getdata(&datalist, &datainfo, ic->interface.name, "month", 2)) {
-		printf("Error: Failed to fetch month data.\n");
-		return;
-	}
-
-	if (datalist->next == NULL) {
-		data_current = datalist;
-	} else {
-		data_previous = datalist;
-		data_current = datalist->next;
-	}
-
-	/* current month */
-	if (data_current->rx + data_current->tx == 0) {
-		rxp = txp = 0;
-	} else {
-		rxp = data_current->rx / (float)(data_current->rx + data_current->tx) * 100;
-		txp = (float)100 - rxp;
-	}
-
-	/* do scaling if needed */
-	if (data_previous != NULL && (data_current->rx + data_current->tx) < (data_previous->rx + data_previous->tx)) {
-		mod = (data_current->rx + data_current->tx) / (float)(data_previous->rx + data_previous->tx);
-		rxp = rxp * mod;
-		txp = txp * mod;
-	}
-
-	/* move graph to center if there's only one to draw for this line */
-	if (data_previous == NULL) {
-		offset = 85;
-	} else {
-		offset = 0;
-	}
-
-	drawdonut(ic, 150 + offset, 163 - headermod, rxp, txp);
-
-	textx = 100 + offset;
-	texty = 118 - headermod;
-
-	d = localtime(&data_current->timestamp);
-	strftime(daytemp, 16, cfg.mformat, d);
-
-	snprintf(buffer, 32, "%*s", getpadding(12, daytemp), daytemp);
-	gdImageString(ic->im, gdFontGetLarge(), textx - 54, texty - 1, (unsigned char *)buffer, ic->ctext);
-
-	if (cfg.summaryrate) {
-		snprintf(buffer, 16, "%15s", gettrafficrate(data_current->rx + data_current->tx, mosecs(data_current->timestamp, ic->interface.updated), 15));
-		gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 58, (unsigned char *)buffer, ic->ctext);
-	} else {
-		texty += 7;
-	}
-
-	snprintf(buffer, 4, "rx ");
-	strncat(buffer, getvalue(data_current->rx, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 18, (unsigned char *)buffer, ic->ctext);
-	snprintf(buffer, 4, "tx ");
-	strncat(buffer, getvalue(data_current->tx, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 30, (unsigned char *)buffer, ic->ctext);
-	snprintf(buffer, 4, " = ");
-	strncat(buffer, getvalue(data_current->rx + data_current->tx, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 44, (unsigned char *)buffer, ic->ctext);
-
-	/* previous month */
-	if (data_previous != NULL) {
-		if (data_previous->rx + data_previous->tx == 0) {
-			rxp = txp = 0;
-		} else {
-			rxp = data_previous->rx / (float)(data_previous->rx + data_previous->tx) * 100;
-			txp = (float)100 - rxp;
-		}
-
-		/* do scaling if needed */
-		if ((data_previous->rx + data_previous->tx) < (data_current->rx + data_current->tx)) {
-			mod = (data_previous->rx + data_previous->tx) / (float)(data_current->rx + data_current->tx);
-			rxp = rxp * mod;
-			txp = txp * mod;
-		}
-
-		drawdonut(ic, 330, 163 - headermod, rxp, txp);
-
-		textx = 280;
-		texty = 118 - headermod;
-
-		d = localtime(&data_previous->timestamp);
-		strftime(daytemp, 16, cfg.mformat, d);
-
-		snprintf(buffer, 32, "%*s", getpadding(12, daytemp), daytemp);
-		gdImageString(ic->im, gdFontGetLarge(), textx - 54, texty - 1, (unsigned char *)buffer, ic->ctext);
-
-		if (cfg.summaryrate) {
-			snprintf(buffer, 16, "%15s", gettrafficrate(data_previous->rx + data_previous->tx, dmonth(d->tm_mon) * 86400, 15));
-			gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 58, (unsigned char *)buffer, ic->ctext);
-		} else {
-			texty += 7;
-		}
-
-		snprintf(buffer, 4, "rx ");
-		strncat(buffer, getvalue(data_previous->rx, 12, RT_Normal), 32);
-		gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 18, (unsigned char *)buffer, ic->ctext);
-		snprintf(buffer, 4, "tx ");
-		strncat(buffer, getvalue(data_previous->tx, 12, RT_Normal), 32);
-		gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 30, (unsigned char *)buffer, ic->ctext);
-		snprintf(buffer, 4, " = ");
-		strncat(buffer, getvalue(data_previous->rx + data_previous->tx, 12, RT_Normal), 32);
-		gdImageString(ic->im, gdFontGetSmall(), textx - 74, texty + 44, (unsigned char *)buffer, ic->ctext);
-	}
-
-	data_current = NULL;
-	data_previous = NULL;
-	dbdatalistfree(&datalist);
-
-	/* all time */
-	textx = 385;
-	texty = 57 - headermod;
-
-	gdImageString(ic->im, gdFontGetLarge(), textx + 12, texty, (unsigned char *)"all time", ic->ctext);
-	snprintf(buffer, 4, "rx ");
-	strncat(buffer, getvalue(ic->interface.rxtotal, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx, texty + 24, (unsigned char *)buffer, ic->ctext);
-	snprintf(buffer, 4, "tx ");
-	strncat(buffer, getvalue(ic->interface.txtotal, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx, texty + 36, (unsigned char *)buffer, ic->ctext);
-	snprintf(buffer, 4, " = ");
-	strncat(buffer, getvalue(ic->interface.rxtotal + ic->interface.txtotal, 12, RT_Normal), 32);
-	gdImageString(ic->im, gdFontGetSmall(), textx, texty + 50, (unsigned char *)buffer, ic->ctext);
-	d = localtime(&ic->interface.created);
-	strftime(datebuff, 16, cfg.tformat, d);
-	snprintf(daytemp, 24, "since %s", datebuff);
-	snprintf(buffer, 32, "%23s", daytemp);
-	gdImageString(ic->im, gdFontGetSmall(), textx - 48, texty + 70, (unsigned char *)buffer, ic->ctext);
-
-	drawlegend(ic, 410, 155 - headermod);
-
-	/* hours if requested */
-	switch (type) {
-		case 1:
-			drawhours(ic, 500, 46 - headermod, rate);
-			break;
-		case 2:
-			drawhours(ic, 12, 215 - headermod, rate);
-			break;
-		default:
-			break;
-	}
 }
 
 void hextorgb(char *input, int *rgb)
