@@ -39,11 +39,15 @@ void drawlegend(IMAGECONTENT *ic, const int x, const int y);
 void drawbar(IMAGECONTENT *ic, const int x, const int y, const int len, const uint64_t rx, const uint64_t tx, const uint64_t max);
 void drawpole(IMAGECONTENT *ic, const int x, const int y, const int len, const uint64_t rx, const uint64_t tx, const uint64_t max);
 void drawdonut(IMAGECONTENT *ic, const int x, const int y, const float rxp, const float txp);
+#ifdef CHECK_VNSTAT
 void drawdonut_libgd_native(IMAGECONTENT *ic, const int x, const int y, const float rxp, const float txp);
-void drawhours(IMAGECONTENT *ic, int x, int y, int rate);
-void drawhourly(IMAGECONTENT *ic, int rate);
+#endif
+int drawhours(IMAGECONTENT *ic, const int x, const int y, const int rate);
+void drawhourly(IMAGECONTENT *ic, const int rate);
 void drawlist(IMAGECONTENT *ic, const char *listname);
-void drawsummary(IMAGECONTENT *ic, int type, int rate);
+void drawsummary(IMAGECONTENT *ic, const int layout, const int rate);
+void drawsummary_alltime(IMAGECONTENT *ic, const int x, const int y);
+void drawsummary_digest(IMAGECONTENT *ic, const int x, const int y, const char *mode);
 void hextorgb(char *input, int *rgb);
 void modcolor(int *rgb, const int offset, const int force);
 char *getimagevalue(const uint64_t b, const int len, const int rate);
