@@ -11,6 +11,7 @@
 /* enable wcswidth on kFreeBSD */
 #if defined(__FreeBSD_kernel__) && defined(__GLIBC__)
 #define __USE_XOPEN
+#define _XOPEN_SOURCE
 #endif
 #include "common.h"
 #include "misc.h"
@@ -127,7 +128,7 @@ uint64_t getbtime(void)
 
 	result = strtoull(statline + 6, (char **)NULL, 0);
 
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__FreeBSD_kernel__)
+#elif defined(BSD_VNSTAT)
 	struct timeval btm;
 	size_t len = sizeof(btm);
 	int mib[2] = {CTL_KERN, KERN_BOOTTIME};
@@ -191,7 +192,7 @@ int getunitspacing(const int len, const int index)
 
 	/* tune spacing according to unit */
 	/* +1 for space between number and unit */
-	l -= strlen(getunitprefix(index)) + 1;
+	l -= (int)strlen(getunitprefix(index)) + 1;
 	if (l < 0) {
 		l = 1;
 	}
@@ -308,7 +309,7 @@ int getratespacing(const int len, const int unitmode, const int unitindex)
 {
 	int l = len;
 
-	l -= strlen(getrateunitprefix(unitmode, unitindex)) + 1;
+	l -= (int)strlen(getrateunitprefix(unitmode, unitindex)) + 1;
 	if (l < 0) {
 		l = 1;
 	}
@@ -394,3 +395,53 @@ int validatedatetime(const char *str)
 
 	return 0;
 }
+
+int issametimeslot(const ListType listtype, const time_t entry, const time_t updated)
+{
+	struct tm e, u;
+
+	if (updated < entry) {
+		return 0;
+	}
+
+	if (entry == updated) {
+		return 1;
+	}
+
+	if (localtime_r(&entry, &e) == NULL || localtime_r(&updated, &u) == NULL) {
+		return 0;
+	}
+
+	switch (listtype) {
+		case LT_5min:
+			if ((entry - (entry % 300)) == (updated - (updated % 300))) {
+				return 1;
+			}
+			break;
+		case LT_Hour:
+			if (e.tm_year == u.tm_year && e.tm_yday == u.tm_yday && e.tm_hour == u.tm_hour) {
+				return 1;
+			}
+			break;
+		case LT_Top:
+		case LT_Day:
+			if (e.tm_year == u.tm_year && e.tm_yday == u.tm_yday) {
+				return 1;
+			}
+			break;
+		case LT_Month:
+			if (e.tm_year == u.tm_year && e.tm_mon == u.tm_mon) {
+				return 1;
+			}
+			break;
+		case LT_Year:
+			if (e.tm_year == u.tm_year) {
+				return 1;
+			}
+			break;
+		case LT_None:
+			return 0;
+	}
+
+	return 0;
+}
