@@ -3,7 +3,7 @@
 #include "misc.h"
 #include "traffic.h"
 
-void trafficmeter(char iface[], unsigned int sampletime)
+void trafficmeter(const char *iface, unsigned int sampletime)
 {
 	/* received bytes packets errs drop fifo frame compressed multicast */
 	/* transmitted bytes packets errs drop fifo colls carrier compressed */
@@ -16,11 +16,13 @@ void trafficmeter(char iface[], unsigned int sampletime)
 		json = 1;
 	}
 
+#ifndef CHECK_VNSTAT
 	/* less than 2 seconds doesn't produce good results */
 	if (sampletime < 2) {
 		printf("Error: Time for sampling too short.\n");
 		exit(EXIT_FAILURE);
 	}
+#endif
 
 	/* read interface info and get values to the first list */
 	if (!getifinfo(iface)) {
@@ -57,6 +59,10 @@ void trafficmeter(char iface[], unsigned int sampletime)
 		sleep(sampletime);
 	}
 
+#ifdef CHECK_VNSTAT
+	sampletime = 1;
+#endif
+
 	/* read those values again... */
 	if (!getifinfo(iface)) {
 		printf("Error: Interface \"%s\" not available, exiting.\n", iface);
@@ -97,7 +103,7 @@ void trafficmeter(char iface[], unsigned int sampletime)
 	}
 }
 
-void livetrafficmeter(char iface[32], int mode)
+void livetrafficmeter(const char *iface, const int mode)
 {
 	/* received bytes packets errs drop fifo frame compressed multicast */
 	/* transmitted bytes packets errs drop fifo colls carrier compressed */
@@ -168,8 +174,10 @@ void livetrafficmeter(char iface[32], int mode)
 
 		timeslept = (uint64_t)time(NULL);
 
+#ifndef CHECK_VNSTAT
 		/* wait 2 seconds for more traffic */
 		sleep(LIVETIME);
+#endif
 
 		timeslept = (uint64_t)time(NULL) - timeslept;
 
@@ -275,10 +283,17 @@ void livetrafficmeter(char iface[32], int mode)
 			printf("}}\n");
 			index++;
 		}
+#ifdef CHECK_VNSTAT
+		break;
+#endif
 	}
 
 	timespent = (uint64_t)time(NULL) - timespent - timeslept;
 
+#ifdef CHECK_VNSTAT
+	timespent = 10;
+#endif
+
 	if (!json) {
 		cursorshow();
 		printf("\n\n");
