@@ -17,11 +17,12 @@ START_TEST(printe_options)
 	strcpy(errorstring, "dummy string");
 	suppress_output();
 	ck_assert_int_eq(printe(PT_Info), 1);
+	ck_assert_int_eq(printe(PT_Warning), 1);
 	ck_assert_int_eq(printe(PT_Error), 1);
 	ck_assert_int_eq(printe(PT_Config), 1);
 	ck_assert_int_eq(printe(PT_Multiline), 1);
 	ck_assert_int_eq(printe(PT_ShortMultiline), 1);
-	ck_assert_int_eq(printe(5), 1);
+	ck_assert_int_eq(printe(6), 1);
 }
 END_TEST
 
@@ -34,11 +35,12 @@ START_TEST(logprint_options)
 	strcpy(cfg.logfile, "/dev/null");
 	strcpy(errorstring, "dummy string");
 	ck_assert_int_eq(logprint(PT_Info), 1);
+	ck_assert_int_eq(logprint(PT_Warning), 1);
 	ck_assert_int_eq(logprint(PT_Error), 1);
 	ck_assert_int_eq(logprint(PT_Config), 1);
 	ck_assert_int_eq(logprint(PT_Multiline), 0);
 	ck_assert_int_eq(logprint(PT_ShortMultiline), 1);
-	ck_assert_int_eq(logprint(5), 1);
+	ck_assert_int_eq(logprint(6), 1);
 }
 END_TEST
 
@@ -138,10 +140,10 @@ START_TEST(mosecs_does_not_change_struct_tm_pointer_content)
 
 	defaultcfg();
 	ck_assert_int_eq(cfg.monthrotate, 1);
-	ck_assert_int_eq(current, timelocal(stm));
+	ck_assert_int_eq(current, mktime(stm));
 	ck_assert_int_ne(mosecs(1, 2), 0);
 	ck_assert_int_ne(mosecs(1, 2), 1);
-	ck_assert_int_eq(current, timelocal(stm));
+	ck_assert_int_eq(current, mktime(stm));
 }
 END_TEST
 
@@ -340,7 +342,7 @@ START_TEST(getversion_returns_a_version)
 }
 END_TEST
 
-START_TEST(timeused_outputs_something_expected)
+START_TEST(timeused_debug_outputs_something_expected_when_debug_is_enabled)
 {
 	int pipe, len;
 	char buffer[512];
@@ -352,13 +354,54 @@ START_TEST(timeused_outputs_something_expected)
 	/* the assumption here is that the next two steps
 	   can always execute in less than one second resulting
 	   in a duration that starts with a zero */
-	timeused("nothing", 1);
-	timeused("something", 0);
+	timeused_debug("that_func", 1);
+	timeused_debug("that_func", 0);
 	fflush(stdout);
 
 	len = (int)read(pipe, buffer, 512);
-	ck_assert_int_gt(len, 0);
-	ck_assert_ptr_ne(strstr(buffer, "something() in 0"), NULL);
+	ck_assert_int_gt(len, 1);
+	ck_assert_ptr_ne(strstr(buffer, "that_func() in 0"), NULL);
+}
+END_TEST
+
+START_TEST(timeused_debug_does_not_output_anything_when_debug_is_disabled)
+{
+	int pipe, len;
+	char buffer[512];
+	memset(&buffer, '\0', sizeof(buffer));
+
+	defaultcfg();
+	debug = 0;
+	pipe = pipe_output();
+	/* the assumption here is that the next two steps
+	   can always execute in less than one second resulting
+	   in a duration that starts with a zero */
+	timeused_debug("other_func", 1);
+	timeused_debug("other_func", 0);
+	printf("-"); // stdout needs to contain something so that read doesn't block
+	fflush(stdout);
+
+	len = (int)read(pipe, buffer, 512);
+	ck_assert_int_eq(len, 1);
+}
+END_TEST
+
+START_TEST(timeused_tracks_used_time)
+{
+	double used;
+	struct timespec ts;
+
+	defaultcfg();
+
+	used = timeused("quick_func", 1);
+	ck_assert(used == 0.0);
+
+	ts.tv_sec = 0;
+	ts.tv_nsec = 100000000; // 0.1 s
+	nanosleep(&ts, NULL);
+
+	used = timeused("quick_func", 0);
+	ck_assert(used > 0.0);
 }
 END_TEST
 
@@ -374,6 +417,8 @@ END_TEST
 void add_common_tests(Suite *s)
 {
 	TCase *tc_common = tcase_create("Common");
+	tcase_add_checked_fixture(tc_common, setup, teardown);
+	tcase_add_unchecked_fixture(tc_common, setup, teardown);
 	tcase_add_test(tc_common, printe_options);
 	tcase_add_test(tc_common, logprint_options);
 	tcase_add_loop_test(tc_common, dmonth_return_within_range, 0, 12);
@@ -399,7 +444,9 @@ void add_common_tests(Suite *s)
 	tcase_add_test(tc_common, isnumeric_it_is);
 	tcase_add_test(tc_common, isnumeric_it_is_not);
 	tcase_add_test(tc_common, getversion_returns_a_version);
-	tcase_add_test(tc_common, timeused_outputs_something_expected);
+	tcase_add_test(tc_common, timeused_debug_outputs_something_expected_when_debug_is_enabled);
+	tcase_add_test(tc_common, timeused_debug_does_not_output_anything_when_debug_is_disabled);
+	tcase_add_test(tc_common, timeused_tracks_used_time);
 	tcase_add_exit_test(tc_common, can_panic, 1);
 	suite_add_tcase(s, tc_common);
 }
