@@ -17,6 +17,15 @@ START_TEST(debugtimestamp_does_not_exit)
 }
 END_TEST
 
+
+START_TEST(initdstate_does_not_crash)
+{
+	DSTATE s;
+	defaultcfg();
+	initdstate(&s);
+}
+END_TEST
+
 START_TEST(addinterfaces_does_nothing_with_no_files)
 {
 	DSTATE s;
@@ -179,18 +188,12 @@ START_TEST(addinterfaces_adds_to_cache_when_running)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
-START_TEST(initdstate_does_not_crash)
-{
-	DSTATE s;
-	defaultcfg();
-	initdstate(&s);
-}
-END_TEST
-
-START_TEST(preparedatabases_exits_with_no_database_dir)
+START_TEST(preparedatabase_exits_with_no_database_dir)
 {
 	DSTATE s;
 
@@ -199,26 +202,26 @@ START_TEST(preparedatabases_exits_with_no_database_dir)
 	suppress_output();
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 
-	preparedatabases(&s);
+	preparedatabase(&s);
 }
 END_TEST
 
-START_TEST(preparedatabases_exits_with_no_databases)
+START_TEST(preparedatabase_exits_with_no_database)
 {
 	DSTATE s;
 
 	defaultcfg();
 	initdstate(&s);
 	suppress_output();
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	ck_assert_int_eq(clean_testdbdir(), 1);
 
-	preparedatabases(&s);
+	preparedatabase(&s);
 }
 END_TEST
 
-START_TEST(preparedatabases_exits_with_no_databases_and_noadd)
+START_TEST(preparedatabase_exits_with_no_database_and_noadd)
 {
 	DSTATE s;
 
@@ -226,15 +229,15 @@ START_TEST(preparedatabases_exits_with_no_databases_and_noadd)
 	initdstate(&s);
 	s.noadd = 1;
 	suppress_output();
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	ck_assert_int_eq(clean_testdbdir(), 1);
 
-	preparedatabases(&s);
+	preparedatabase(&s);
 }
 END_TEST
 
-START_TEST(preparedatabases_with_no_databases_creates_databases)
+START_TEST(preparedatabase_with_no_database_creates_database)
 {
 	int ret;
 	DSTATE s;
@@ -242,7 +245,7 @@ START_TEST(preparedatabases_with_no_databases_creates_databases)
 	defaultcfg();
 	initdstate(&s);
 	suppress_output();
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	ck_assert_int_eq(clean_testdbdir(), 1);
 	fake_proc_net_dev("w", "ethone", 1, 2, 3, 4);
@@ -252,7 +255,7 @@ START_TEST(preparedatabases_with_no_databases_creates_databases)
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
-	preparedatabases(&s);
+	preparedatabase(&s);
 
 	ck_assert_int_eq(db_getinterfacecountbyname("ethone"), 1);
 	ck_assert_int_eq(db_getinterfacecountbyname("ethtwo"), 1);
@@ -281,7 +284,7 @@ START_TEST(filldatabaselist_exits_with_no_database_dir)
 	defaultcfg();
 	initdstate(&s);
 	disable_logprints();
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 
 	filldatabaselist(&s);
@@ -317,7 +320,7 @@ START_TEST(filldatabaselist_adds_databases)
 	initdstate(&s);
 	disable_logprints();
 	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	s.sync = 1;
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	ck_assert_int_eq(clean_testdbdir(), 1);
@@ -349,6 +352,8 @@ START_TEST(filldatabaselist_adds_databases)
 	ck_assert_int_eq(intsignal, 42);
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -381,6 +386,8 @@ START_TEST(adjustsaveinterval_with_filled_cache)
 	adjustsaveinterval(&s);
 
 	ck_assert_int_eq(s.saveinterval, cfg.saveinterval * 60);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -461,6 +468,7 @@ START_TEST(processdatacache_can_process_things)
 	s.bootdetected = 1;
 	s.dodbsave = 1;
 	s.cleanuphour = getcurrenthour() + 1;
+	s.dbifcount = 3;
 
 	ck_assert_int_eq(datacache_count(&s.dcache), 0);
 	ret = datacache_add(&s.dcache, "ethnotindb", 0);
@@ -489,6 +497,8 @@ START_TEST(processdatacache_can_process_things)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -600,7 +610,7 @@ START_TEST(handleintsignals_handles_sighup)
 
 	s.running = 1;
 	intsignal = SIGHUP;
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	handleintsignals(&s);
 	ck_assert_int_eq(intsignal, 0);
 	ck_assert_int_eq(s.running, 1);
@@ -620,7 +630,7 @@ START_TEST(preparedirs_with_no_dir)
 	defaultcfg();
 	cfg.uselogging = 1;
 	s.rundaemon = 1;
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	snprintf(logdir, 500, "%s/log/vnstat", TESTDIR);
 	snprintf(piddir, 500, "%s/pid/vnstat", TESTDIR);
 	snprintf(cfg.logfile, 512, "%s/vnstat.log", logdir);
@@ -646,7 +656,7 @@ START_TEST(preparedirs_with_dir)
 	defaultcfg();
 	cfg.uselogging = 1;
 	s.rundaemon = 1;
-	strncpy_nt(s.dirname, TESTDBDIR, 512);
+	strncpy_nt(cfg.dbdir, TESTDBDIR, 512);
 	snprintf(logdir, 500, "%s/log/vnstat", TESTDIR);
 	snprintf(piddir, 500, "%s/pid/vnstat", TESTDIR);
 	snprintf(cfg.logfile, 512, "%s/vnstat.log", logdir);
@@ -702,13 +712,15 @@ START_TEST(interfacechangecheck_with_no_changes_in_iflist)
 	ck_assert_int_eq(remove_directory(TESTDIR), 1);
 	fake_proc_net_dev("w", "ethsomething", 1, 2, 3, 4);
 	fake_proc_net_dev("a", "ethelse", 5, 6, 7, 8);
-	ck_assert_int_ne(getiflist(&ifacelist, 0), 0);
+	ck_assert_int_ne(getifliststring(&ifacelist, 0), 0);
 	ifhash = simplehash(ifacelist, (int)strlen(ifacelist));
 	s.iflisthash = ifhash;
 
 	interfacechangecheck(&s);
 	ck_assert_int_eq(s.iflisthash, ifhash);
 	ck_assert_int_eq(s.forcesave, 0);
+
+        free(ifacelist);
 }
 END_TEST
 
@@ -747,6 +759,8 @@ START_TEST(interfacechangecheck_with_filled_cache)
 	ck_assert_int_eq(s.forcesave, 1);
 	ck_assert_int_eq(datacache_count(&s.dcache), 2);
 	ck_assert_int_eq(datacache_activecount(&s.dcache), 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -779,6 +793,8 @@ START_TEST(initcachevalues_does_not_init_without_database)
 
 	ret = initcachevalues(&s, &s.dcache);
 	ck_assert_int_eq(ret, 0);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -814,6 +830,8 @@ START_TEST(initcachevalues_does_init)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -886,6 +904,8 @@ START_TEST(waittimesync_does_not_wait_with_new_interfaces)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -920,6 +940,8 @@ START_TEST(waittimesync_knows_when_to_wait)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -954,6 +976,8 @@ START_TEST(waittimesync_knows_when_to_give_up)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1072,30 +1096,7 @@ START_TEST(handledatabaseerror_exits_on_fatal_error)
 }
 END_TEST
 
-START_TEST(handledatabaseerror_does_not_exit_if_limit_is_not_reached)
-{
-	int i;
-	DSTATE s;
-	defaultcfg();
-	initdstate(&s);
-	disable_logprints();
-
-	ck_assert_int_eq(s.dbretrycount, 0);
-
-	db_errcode = SQLITE_BUSY;
-	handledatabaseerror(&s);
-
-	ck_assert_int_eq(s.dbretrycount, 1);
-
-	for (i = 1; i < DBRETRYLIMIT - 1; i++) {
-		handledatabaseerror(&s);
-	}
-
-	ck_assert_int_eq(s.dbretrycount, DBRETRYLIMIT - 1);
-}
-END_TEST
-
-START_TEST(handledatabaseerror_exits_if_limit_is_reached)
+START_TEST(handledatabaseerror_does_not_exit_if_limit_is_not_exceeded)
 {
 	int i;
 	DSTATE s;
@@ -1113,6 +1114,29 @@ START_TEST(handledatabaseerror_exits_if_limit_is_reached)
 	for (i = 1; i < DBRETRYLIMIT; i++) {
 		handledatabaseerror(&s);
 	}
+
+	ck_assert_int_eq(s.dbretrycount, DBRETRYLIMIT);
+}
+END_TEST
+
+START_TEST(handledatabaseerror_exits_if_limit_is_exceeded)
+{
+	int i;
+	DSTATE s;
+	defaultcfg();
+	initdstate(&s);
+	disable_logprints();
+
+	ck_assert_int_eq(s.dbretrycount, 0);
+
+	db_errcode = SQLITE_BUSY;
+	handledatabaseerror(&s);
+
+	ck_assert_int_eq(s.dbretrycount, 1);
+
+	for (i = 1; i < DBRETRYLIMIT + 1; i++) {
+		handledatabaseerror(&s);
+	}
 }
 END_TEST
 
@@ -1158,6 +1182,8 @@ START_TEST(cleanremovedinterfaces_allows_interfaces_to_be_removed)
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1184,6 +1210,8 @@ START_TEST(processifinfo_syncs_when_needed)
 	ck_assert_int_eq(s.dcache->syncneeded, 0);
 	ck_assert_int_eq(s.dcache->currx, 11);
 	ck_assert_int_eq(s.dcache->curtx, 22);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1212,6 +1240,8 @@ START_TEST(processifinfo_skips_update_if_timestamps_make_no_sense)
 	ck_assert_int_eq(s.dcache->syncneeded, 0);
 	ck_assert_int_eq(s.dcache->currx, 0);
 	ck_assert_int_eq(s.dcache->curtx, 0);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1265,6 +1295,8 @@ START_TEST(processifinfo_syncs_if_timestamps_match)
 	ck_assert_int_eq(s.dcache->currx, 11);
 	ck_assert_int_eq(s.dcache->curtx, 22);
 	ck_assert_ptr_eq(s.dcache->log, NULL);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1299,6 +1331,8 @@ START_TEST(processifinfo_adds_traffic)
 	ck_assert_int_eq(s.dcache->currx, 11);
 	ck_assert_int_eq(s.dcache->curtx, 22);
 	ck_assert_ptr_ne(s.dcache->log, NULL);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1334,6 +1368,8 @@ START_TEST(processifinfo_does_not_add_traffic_when_over_limit)
 	ck_assert_int_eq(s.dcache->currx, 1111111);
 	ck_assert_int_eq(s.dcache->curtx, 2222222);
 	ck_assert_ptr_eq(s.dcache->log, NULL);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1371,6 +1407,8 @@ START_TEST(processifinfo_adds_zero_traffic_when_over_limit)
 	ck_assert_ptr_ne(s.dcache->log, NULL);
 	ck_assert_int_eq(s.dcache->log->rx, 0);
 	ck_assert_int_eq(s.dcache->log->tx, 0);
+
+	datacache_clear(&s.dcache);
 }
 END_TEST
 
@@ -1409,6 +1447,8 @@ START_TEST(datacache_status_can_show_limits)
 	ck_assert_int_eq(ret, 1);
 
 	datacache_status(&dcache);
+
+	datacache_clear(&dcache);
 }
 END_TEST
 
@@ -1430,22 +1470,26 @@ START_TEST(datacache_status_has_no_issues_with_large_number_of_interfaces)
 	}
 
 	datacache_status(&dcache);
+
+	datacache_clear(&dcache);
 }
 END_TEST
 
 void add_daemon_tests(Suite *s)
 {
 	TCase *tc_daemon = tcase_create("Daemon");
+	tcase_add_checked_fixture(tc_daemon, setup, teardown);
+	tcase_add_unchecked_fixture(tc_daemon, setup, teardown);
 	tcase_add_test(tc_daemon, debugtimestamp_does_not_exit);
 	tcase_add_test(tc_daemon, initdstate_does_not_crash);
 	tcase_add_test(tc_daemon, addinterfaces_does_nothing_with_no_files);
 	tcase_add_test(tc_daemon, addinterfaces_adds_interfaces);
 	tcase_add_test(tc_daemon, addinterfaces_adds_only_new_interfaces);
 	tcase_add_test(tc_daemon, addinterfaces_adds_to_cache_when_running);
-	tcase_add_exit_test(tc_daemon, preparedatabases_exits_with_no_database_dir, 1);
-	tcase_add_exit_test(tc_daemon, preparedatabases_exits_with_no_databases, 1);
-	tcase_add_exit_test(tc_daemon, preparedatabases_exits_with_no_databases_and_noadd, 1);
-	tcase_add_test(tc_daemon, preparedatabases_with_no_databases_creates_databases);
+	tcase_add_exit_test(tc_daemon, preparedatabase_exits_with_no_database_dir, 1);
+	tcase_add_exit_test(tc_daemon, preparedatabase_exits_with_no_database, 1);
+	tcase_add_exit_test(tc_daemon, preparedatabase_exits_with_no_database_and_noadd, 1);
+	tcase_add_test(tc_daemon, preparedatabase_with_no_database_creates_database);
 	tcase_add_test(tc_daemon, setsignaltraps_does_not_exit);
 	tcase_add_exit_test(tc_daemon, filldatabaselist_exits_with_no_database_dir, 1);
 	tcase_add_test(tc_daemon, filldatabaselist_does_not_exit_with_empty_database_dir);
@@ -1483,8 +1527,8 @@ void add_daemon_tests(Suite *s)
 	tcase_add_test(tc_daemon, detectboot_sets_btime_for_new_database);
 	tcase_add_test(tc_daemon, detectboot_can_detect_boot);
 	tcase_add_exit_test(tc_daemon, handledatabaseerror_exits_on_fatal_error, 1);
-	tcase_add_test(tc_daemon, handledatabaseerror_does_not_exit_if_limit_is_not_reached);
-	tcase_add_exit_test(tc_daemon, handledatabaseerror_exits_if_limit_is_reached, 1);
+	tcase_add_test(tc_daemon, handledatabaseerror_does_not_exit_if_limit_is_not_exceeded);
+	tcase_add_exit_test(tc_daemon, handledatabaseerror_exits_if_limit_is_exceeded, 1);
 	tcase_add_test(tc_daemon, cleanremovedinterfaces_allows_interfaces_to_be_removed);
 	tcase_add_test(tc_daemon, processifinfo_syncs_when_needed);
 	tcase_add_test(tc_daemon, processifinfo_skips_update_if_timestamps_make_no_sense);
