@@ -2,6 +2,7 @@
 #include "vnstat_tests.h"
 #include "dbsql_tests.h"
 #include "dbsql.h"
+#include "misc.h"
 #include "cfg.h"
 
 START_TEST(db_close_does_no_harm_when_db_is_already_closed)
@@ -177,7 +178,7 @@ START_TEST(db_addtraffic_can_add_traffic_and_interfaces)
 }
 END_TEST
 
-START_TEST(db_addtraffic_dated_does_not_turn_back_time)
+START_TEST(db_addtraffic_dated_does_not_touch_updated_time)
 {
 	int ret;
 	interfaceinfo info;
@@ -211,7 +212,7 @@ START_TEST(db_addtraffic_dated_does_not_turn_back_time)
 	ck_assert_int_eq(ret, 1);
 	ck_assert_int_eq(info.rxtotal, 3);
 	ck_assert_int_eq(info.txtotal, 3);
-	ck_assert_int_eq(info.updated, 2100000000);
+	ck_assert_int_lt(info.updated, 2100000000);
 
 	ret = db_addtraffic_dated("eth0", 1, 1, 1000);
 	ck_assert_int_eq(ret, 1);
@@ -219,7 +220,7 @@ START_TEST(db_addtraffic_dated_does_not_turn_back_time)
 	ck_assert_int_eq(ret, 1);
 	ck_assert_int_eq(info.rxtotal, 4);
 	ck_assert_int_eq(info.txtotal, 4);
-	ck_assert_int_eq(info.updated, 2100000000);
+	ck_assert_int_lt(info.updated, 2100000000);
 
 	ret = db_addtraffic("eth0", 1, 1);
 	ck_assert_int_eq(ret, 1);
@@ -487,6 +488,73 @@ START_TEST(db_removeinterface_can_remove_interfaces)
 }
 END_TEST
 
+START_TEST(db_renameinterface_knows_if_interface_exists)
+{
+	int ret;
+
+	defaultcfg();
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_renameinterface("eth0", "eth1");
+	ck_assert_int_eq(ret, 0);
+	ret = db_renameinterface("nothing", "something");
+	ck_assert_int_eq(ret, 0);
+	ret = db_renameinterface("", "");
+	ck_assert_int_eq(ret, 0);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(db_renameinterface_can_rename_interfaces)
+{
+	int ret;
+
+	defaultcfg();
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addinterface("eth0");
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("eth1");
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("eth2");
+	ck_assert_int_eq(ret, 1);
+
+	ck_assert_int_eq(db_getinterfacecount(), 3);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth0"), 1);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth1"), 1);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth2"), 1);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth3"), 0);
+
+	suppress_output();
+
+	ret = db_renameinterface("eth0", "eth1");
+	ck_assert_int_eq(ret, 0);
+	ret = db_renameinterface("eth2", "eth3");
+	ck_assert_int_eq(ret, 1);
+	ret = db_renameinterface("eth1", "eth2");
+	ck_assert_int_eq(ret, 1);
+	ret = db_renameinterface("eth0", "");
+	ck_assert_int_eq(ret, 0);
+	ret = db_renameinterface("eth0", "eth1");
+	ck_assert_int_eq(ret, 1);
+
+	ck_assert_int_eq(db_getinterfacecount(), 3);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth0"), 0);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth1"), 1);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth2"), 1);
+	ck_assert_int_eq(db_getinterfacecountbyname("eth3"), 1);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 START_TEST(db_getinterfacecount_counts_interfaces)
 {
 	int ret;
@@ -695,8 +763,7 @@ END_TEST
 START_TEST(db_getiflist_lists_interfaces)
 {
 	int ret;
-	dbiflist *dbifl = NULL, *dbifl_i = NULL;
-	;
+	iflist *dbifl = NULL, *dbifl_i = NULL;
 
 	defaultcfg();
 
@@ -734,7 +801,7 @@ START_TEST(db_getiflist_lists_interfaces)
 	ck_assert_str_eq(dbifl_i->interface, "eth1");
 	ck_assert_ptr_eq(dbifl_i->next, NULL);
 
-	dbiflistfree(&dbifl);
+	iflistfree(&dbifl);
 	ck_assert_ptr_eq(dbifl, NULL);
 }
 END_TEST
@@ -883,7 +950,6 @@ START_TEST(db_validate_with_valid_version)
 	int ret;
 	defaultcfg();
 	suppress_output();
-	debug = 1;
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2186,6 +2252,8 @@ END_TEST
 void add_dbsql_tests(Suite *s)
 {
 	TCase *tc_dbsql = tcase_create("DB SQL");
+	tcase_add_checked_fixture(tc_dbsql, setup, teardown);
+	tcase_add_unchecked_fixture(tc_dbsql, setup, teardown);
 	tcase_add_test(tc_dbsql, db_close_does_no_harm_when_db_is_already_closed);
 	tcase_add_test(tc_dbsql, db_open_rw_can_create_database_if_file_does_not_exist);
 	tcase_add_test(tc_dbsql, db_open_ro_cannot_create_a_database);
@@ -2198,7 +2266,7 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_setinfo_can_not_update_nonexisting_name);
 	tcase_add_test(tc_dbsql, db_addtraffic_with_no_traffic_does_nothing);
 	tcase_add_test(tc_dbsql, db_addtraffic_can_add_traffic_and_interfaces);
-	tcase_add_test(tc_dbsql, db_addtraffic_dated_does_not_turn_back_time);
+	tcase_add_test(tc_dbsql, db_addtraffic_dated_does_not_touch_updated_time);
 	tcase_add_test(tc_dbsql, db_getinterfacecount_counts_interfaces);
 	tcase_add_test(tc_dbsql, db_getinterfacecountbyname_counts_interfaces);
 	tcase_add_test(tc_dbsql, db_setactive_fails_with_no_open_db);
@@ -2215,6 +2283,8 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_addinterface_can_not_add_same_interface_twice);
 	tcase_add_test(tc_dbsql, db_removeinterface_knows_if_interface_exists);
 	tcase_add_test(tc_dbsql, db_removeinterface_can_remove_interfaces);
+	tcase_add_test(tc_dbsql, db_renameinterface_knows_if_interface_exists);
+	tcase_add_test(tc_dbsql, db_renameinterface_can_rename_interfaces);
 	tcase_add_test(tc_dbsql, db_getcounters_with_no_interface);
 	tcase_add_test(tc_dbsql, db_setcounters_with_no_interface);
 	tcase_add_test(tc_dbsql, db_interface_info_manipulation);
