@@ -69,9 +69,17 @@ END_TEST
 START_TEST(setuser_with_current_user)
 {
 	char temp[16];
-	suppress_output();
-	snprintf(temp, 16, "%d", getuser(""));
-	setuser(temp);
+	struct passwd *pw;
+
+	pw = getpwuid(getuid());
+
+	if (pw != NULL) {
+		suppress_output();
+		snprintf(temp, 16, "%d", getuser(""));
+		setuser(temp);
+	} else {
+		printf("Note: skipping test setuser_with_current_user due to non-existing user\n");
+	}
 }
 END_TEST
 
@@ -85,15 +93,25 @@ END_TEST
 START_TEST(setgroup_with_current_group)
 {
 	char temp[16];
-	suppress_output();
-	snprintf(temp, 16, "%d", getgroup(""));
-	setgroup(temp);
+	struct group *gr;
+
+	gr = getgrgid(getgid());
+
+	if (gr != NULL) {
+		suppress_output();
+		snprintf(temp, 16, "%d", getgroup(""));
+		setgroup(temp);
+	} else {
+		printf("Note: skipping test setgroup_with_current_group due to non-existing group\n");
+	}
 }
 END_TEST
 
 void add_id_tests(Suite *s)
 {
 	TCase *tc_id = tcase_create("ID");
+	tcase_add_checked_fixture(tc_id, setup, teardown);
+	tcase_add_unchecked_fixture(tc_id, setup, teardown);
 	tcase_add_test(tc_id, getuser_root_string);
 	tcase_add_test(tc_id, getuser_root_numeric);
 	tcase_add_exit_test(tc_id, getuser_no_such_user_string, 1);
