@@ -10,17 +10,24 @@
 #include "datacache_tests.h"
 #include "fs_tests.h"
 #include "id_tests.h"
+#include "iflist_tests.h"
+#include "cli_tests.h"
+#include "parseargs_tests.h"
 #if defined(HAVE_IMAGE)
 #include "image_tests.h"
 #endif
 
+int output_suppressed = 0;
+
 int main(void)
 {
+	Suite *s;
+	SRunner *sr;
 	int number_failed = 0;
-	debug = 0;
 
-	Suite *s = test_suite();
-	SRunner *sr = srunner_create(s);
+	verify_fork_status();
+	s = test_suite();
+	sr = srunner_create(s);
 	srunner_set_log(sr, "test.log");
 	srunner_set_xml(sr, "test.xml");
 	srunner_run_all(sr, CK_NORMAL);
@@ -48,6 +55,9 @@ Suite *test_suite(void)
 	add_datacache_tests(s);
 	add_fs_tests(s);
 	add_id_tests(s);
+	add_iflist_tests(s);
+	add_cli_tests(s);
+	add_parseargs_tests(s);
 #if defined(HAVE_IMAGE)
 	add_image_tests(s);
 #endif
@@ -55,9 +65,33 @@ Suite *test_suite(void)
 	return s;
 }
 
+/* tests can't reliably be executed if check doesn't have forking enabled */
+/* and only SRunner knows that so a dummy runner needs to be created */
+void verify_fork_status(void)
+{
+	Suite *s = suite_create("fork status check");
+	SRunner *sr = srunner_create(s);
+	if (srunner_fork_status(sr) == CK_NOFORK) {
+		printf("Error: Tests require Check to have fork mode enabled.\n");
+		exit(1);
+	}
+	srunner_free(sr);
+}
+
+void setup(void) {
+	debug = 0;
+}
+
+void teardown(void) {
+	;
+}
+
 void suppress_output(void)
 {
-	fclose(stdout);
+	if (!output_suppressed) {
+		fclose(stdout);
+		output_suppressed = 1;
+	}
 }
 
 int pipe_output(void)
@@ -68,9 +102,12 @@ int pipe_output(void)
 		ck_abort_msg("error \"%s\" while creating pipe", strerror(errno));
 	}
 
-	dup2(out_pipe[1], STDOUT_FILENO);
-	close(out_pipe[1]);
+	if (out_pipe[1] != STDOUT_FILENO) {
+		dup2(out_pipe[1], STDOUT_FILENO);
+		close(out_pipe[1]);
+	}
 
+	output_suppressed = 1;
 	return out_pipe[0];
 }
 
