@@ -3,8 +3,14 @@
 
 #include <check.h>
 
+extern int output_suppressed;
+
 Suite *test_suite(void);
+void verify_fork_status(void);
+void setup(void);
+void teardown(void);
 void suppress_output(void);
+void restore_output(void);
 int pipe_output(void);
 void disable_logprints(void);
 int clean_testdbdir(void);
